@@ -14,6 +14,7 @@
 
 // has to be first
 #include "openmpt/all/BuildSettings.hpp"
+#include "openmpt/all/PlatformFixes.hpp"
 
 
 #if defined(MODPLUG_TRACKER)
@@ -32,10 +33,10 @@
 #include <afxcview.h>
 // cppcheck-suppress missingInclude
 #include <afxdlgs.h>
-#ifdef MPT_MFC_FULL
+#ifndef _AFX_NO_MFC_CONTROLS_IN_DIALOGS
 // cppcheck-suppress missingInclude
 #include <afxlistctrl.h>
-#endif // MPT_MFC_FULL
+#endif // !_AFX_NO_MFC_CONTROLS_IN_DIALOGS
 // cppcheck-suppress missingInclude
 #include <afxole.h>
 
@@ -58,16 +59,21 @@
 #endif
 
 
-#include "mpt/base/span.hpp"
+#include "mpt/check/compiler.hpp"
 #include "mpt/check/libc.hpp"
+#include "mpt/check/libcxx.hpp"
 #if defined(MPT_WITH_MFC)
 #include "mpt/check/mfc.hpp"
 #endif
 #if MPT_OS_WINDOWS
 #include "mpt/check/windows.hpp"
 #endif
-#include "mpt/exception_text/exception_text.hpp"
+
+#include "mpt/base/span.hpp"
+#include "mpt/exception/exception.hpp"
+#include "mpt/exception/exception_text.hpp"
 #include "mpt/out_of_memory/out_of_memory.hpp"
+#include "mpt/string/types.hpp"
 #include "mpt/system_error/system_error.hpp"
 
 #include "openmpt/base/Types.hpp"
