@@ -10,12 +10,20 @@
 #include "stdafx.h"
 #include "version.h"
 
+#include "mpt/format/join.hpp"
+#include "mpt/parse/parse.hpp"
+#include "mpt/string/utility.hpp"
+
 #include "mptString.h"
 #include "mptStringFormat.h"
-#include "mptStringParse.h" 
 
 #include "versionNumber.h"
+
+#if __has_include("svn_version.h")
 #include "svn_version.h"
+#else
+#include "../build/svn_version/svn_version.h"
+#endif
 
 
 
@@ -47,10 +55,10 @@ mpt::ustring Version::GetOpenMPTVersionString() const
 Version Version::Parse(const mpt::ustring &s)
 {
 	uint32 result = 0;
-	std::vector<mpt::ustring> numbers = mpt::String::Split<mpt::ustring>(s, U_("."));
+	std::vector<mpt::ustring> numbers = mpt::split(s, U_("."));
 	for (std::size_t i = 0; i < numbers.size() && i < 4; ++i)
 	{
-		result |= (mpt::String::Parse::Hex<unsigned int>(numbers[i]) & 0xff) << ((3 - i) * 8);
+		result |= (mpt::parse_hex<unsigned int>(numbers[i]) & 0xff) << ((3 - i) * 8);
 	}
 	return Version(result);
 }
@@ -137,7 +145,7 @@ static int GetRevision()
 		{
 			svnversion = svnversion.substr(0, svnversion.find("P"));
 		}
-		return ConvertStrTo<int>(svnversion);
+		return mpt::parse<int>(svnversion);
 	#else
 		MPT_WARNING_STATEMENT("SVN revision unknown. Please check your build system.");
 		return 0;
@@ -273,14 +281,14 @@ VersionWithRevision VersionWithRevision::Parse(const mpt::ustring &s)
 {
 	Version version = Version::Parse(mpt::ustring());
 	uint64 revision = 0;
-	const auto tokens = mpt::String::Split<mpt::ustring>(s, U_("-"));
+	const auto tokens = mpt::split(s, U_("-"));
 	if(tokens.size() >= 1)
 	{
 		version = Version::Parse(tokens[0]);
 	}
 	if(tokens.size() >= 2)
 	{
-		revision = ConvertStrTo<uint64>(tokens[1].substr(1));
+		revision = mpt::parse<uint64>(tokens[1].substr(1));
 	}
 	return {version, revision};
 }
@@ -513,7 +521,7 @@ mpt::ustring GetVersionString(FlagSet<Build::Strings> strings)
 	{
 		result.push_back(GetBuildFeaturesString());
 	}
-	return mpt::trim(mpt::String::Combine<mpt::ustring>(result, U_("")));
+	return mpt::trim(mpt::join_format<mpt::ustring>(result, U_("")));
 }
 
 mpt::ustring GetVersionStringPure()
@@ -592,12 +600,12 @@ mpt::ustring GetFullCreditsString()
 		"libopenmpt (based on OpenMPT / Open ModPlug Tracker)\n"
 #endif
 		"\n"
-		"Copyright \xC2\xA9 2004-2023 OpenMPT Project Developers and Contributors\n"
+		"Copyright \xC2\xA9 2004-2025 OpenMPT Project Developers and Contributors\n"
 		"Copyright \xC2\xA9 1997-2003 Olivier Lapicque\n"
 		"\n"
 		"Developers:\n"
-		"Johannes Schultz (2008-2023)\n"
-		"J\xC3\xB6rn Heusipp (2012-2023)\n"
+		"Johannes Schultz (2008-2025)\n"
+		"J\xC3\xB6rn Heusipp (2012-2025)\n"
 		"Ahti Lepp\xC3\xA4nen (2005-2011)\n"
 		"Robin Fernandes (2004-2007)\n"
 		"Sergiy Pylypenko (2007)\n"
@@ -607,10 +615,13 @@ mpt::ustring GetFullCreditsString()
 		"\n"
 		"Additional contributors:\n"
 		"coda (https://coda.s3m.us/)\n"
+		"cs127 (https://cs127.github.io/)\n"
 		"Jo\xC3\xA3o Baptista de Paula e Silva (https://joaobapt.com/)\n"
 		"kode54 (https://kode54.net/)\n"
 		"Revenant (https://revenant1.net/)\n"
+		"SYRiNX\n"
 		"xaimus (http://xaimus.com/)\n"
+		"zersal\n"
 		"\n"
 		"Thanks to:\n"
 		"\n"
@@ -618,15 +629,12 @@ mpt::ustring GetFullCreditsString()
 		"http://modplug-xmms.sourceforge.net/\n"
 		"\n"
 #ifdef MODPLUG_TRACKER
-		"Stephan M. Bernsee for pitch shifting source code\n"
-		"http://www.dspdimension.com/\n"
+		"Geraint Luff for Signalsmith Stretch\n"
+		"https://signalsmith-audio.co.uk/code/stretch/\n"
 		"\n"
 		"Aleksey Vaneev of Voxengo for r8brain sample rate converter\n"
 		"https://github.com/avaneev/r8brain-free-src\n"
 		"\n"
-		"Olli Parviainen for SoundTouch Library (time stretching)\n"
-		"https://www.surina.net/soundtouch/\n"
-		"\n"
 #endif
 #ifdef MPT_WITH_VST
 		"Hermann Seib for his example VST Host implementation\n"
@@ -650,7 +658,7 @@ mpt::ustring GetFullCreditsString()
 		"https://github.com/iamgreaser/it2everything/\n"
 		"\n"
 		"Antti S. Lankila for Amiga resampler implementation\n"
-		"https://bel.fi/alankila/modguide/interpolate.txt\n"
+		"https://web.archive.org/web/20221228071135/https://bel.fi/alankila/modguide/\n"
 		"\n"
 		"Shayde / Reality Productions for Opal OPL3 emulator\n"
 		"https://www.3eality.com/\n"
@@ -764,18 +772,18 @@ mpt::ustring GetFullCreditsString()
 		"https://www.behance.net/ulfurkolka\n"
 		"\n"
 		"Nobuyuki for file icon\n"
-		"https://twitter.com/nobuyukinyuu\n"
+		"https://github.com/nobuyukinyuu/\n"
 		"\n"
 #endif
 		"Daniel Collin (emoon/TBL) for providing test infrastructure\n"
-		"https://twitter.com/daniel_collin\n"
+		"https://mastodon.gamedev.place/@daniel_collin\n"
 		"\n"
-		"The people at ModPlug forums for crucial contribution\n"
+		"The people in the ModPlug community for crucial contribution\n"
 		"in the form of ideas, testing and support;\n"
 		"thanks particularly to:\n"
-		"33, 8bitbubsy, Anboi, BooT-SectoR-ViruZ, Bvanoudtshoorn\n"
-		"christofori, cubaxd, Diamond, Ganja, Georg, Goor00,\n"
-		"Harbinger, jmkz, KrazyKatz, LPChip, Nofold, Rakib, Sam Zen\n"
+		"33, 8bitbubsy, AliceLR, Anboi, BooT-SectoR-ViruZ, Bvanoudtshoorn\n"
+		"a11cf0, christofori, cubaxd, Diamond, Ganja, Georg, Goor00,\n"
+		"Harbinger, jmkz, KrazyKatz, LPChip, MiDoRi, Nofold, Rakib, Sam Zen\n"
 		"Skaven, Skilletaudio, Snu, Squirrel Havoc, Teimoso, Waxhead\n"
 		"\n"
 #ifdef MPT_WITH_VST
@@ -792,7 +800,7 @@ mpt::ustring GetFullCreditsString()
 mpt::ustring GetLicenseString()
 {
 	return MPT_UTF8(
-		"Copyright (c) 2004-2023, OpenMPT Project Developers and Contributors" "\n"
+		"Copyright (c) 2004-2025, OpenMPT Project Developers and Contributors" "\n"
 		"Copyright (c) 1997-2003, Olivier Lapicque" "\n"
 		"All rights reserved." "\n"
 		"" "\n"
