@@ -21,12 +21,15 @@
 OPENMPT_NAMESPACE_BEGIN
 
 
+#if !defined(MPT_WITH_ANCIENT)
+
+
 struct PPBITBUFFER
 {
-	uint32 bitcount;
-	uint32 bitbuffer;
-	const uint8 *pStart;
-	const uint8 *pSrc;
+	uint32 bitcount = 0;
+	uint32 bitbuffer = 0;
+	const uint8 *pStart = nullptr;
+	const uint8 *pSrc = nullptr;
 
 	uint32 GetBits(uint32 n);
 };
@@ -36,15 +39,16 @@ uint32 PPBITBUFFER::GetBits(uint32 n)
 {
 	uint32 result = 0;
 
-	for (uint32 i=0; i<n; i++)
+	for(uint32 i = 0; i < n; i++)
 	{
-		if (!bitcount)
+		if(!bitcount)
 		{
 			bitcount = 8;
-			if (pSrc != pStart) pSrc--;
+			if(pSrc != pStart)
+				pSrc--;
 			bitbuffer = *pSrc;
 		}
-		result = (result<<1) | (bitbuffer&1);
+		result = (result << 1) | (bitbuffer & 1);
 		bitbuffer >>= 1;
 		bitcount--;
 	}
@@ -52,57 +56,54 @@ uint32 PPBITBUFFER::GetBits(uint32 n)
 }
 
 
-static bool PP20_DoUnpack(const uint8 *pSrc, uint32 nSrcLen, uint8 *pDst, uint32 nDstLen)
+static bool PP20_DoUnpack(mpt::span<const uint8> src, uint8 *pDst, uint32 dstLen)
 {
+	const std::array<uint8, 4> modeTable{src[0], src[1], src[2], src[3]};
 	PPBITBUFFER BitBuffer;
-	uint32 nBytesLeft;
-
-	BitBuffer.pStart = pSrc;
-	BitBuffer.pSrc = pSrc + nSrcLen - 4;
-	BitBuffer.bitbuffer = 0;
-	BitBuffer.bitcount = 0;
-	BitBuffer.GetBits(pSrc[nSrcLen-1]);
-	nBytesLeft = nDstLen;
-	while (nBytesLeft > 0)
+	BitBuffer.pStart = src.data();
+	BitBuffer.pSrc = src.data() + src.size() - 4;
+	BitBuffer.GetBits(src.data()[src.size() - 1]);
+	uint32 bytesLeft = dstLen;
+	while(bytesLeft > 0)
 	{
-		if (!BitBuffer.GetBits(1))
+		if(!BitBuffer.GetBits(1))
 		{
-			uint32 n = 1, code;
+			uint32 count = 1, countAdd;
 			do
 			{
-				code = BitBuffer.GetBits(2);
-				n += code;
-			} while(code == 3);
-			LimitMax(n, nBytesLeft);
-			for (uint32 i=0; i<n; i++)
+				countAdd = BitBuffer.GetBits(2);
+				count += countAdd;
+			} while(countAdd == 3);
+			LimitMax(count, bytesLeft);
+			for(uint32 i = 0; i < count; i++)
 			{
-				pDst[--nBytesLeft] = (uint8)BitBuffer.GetBits(8);
+				pDst[--bytesLeft] = (uint8)BitBuffer.GetBits(8);
 			}
-			if (!nBytesLeft) break;
+			if(!bytesLeft)
+				break;
 		}
 		{
-			uint32 n = BitBuffer.GetBits(2)+1;
-			if(n < 1 || n-1 >= nSrcLen) return false;
-			uint32 nbits = pSrc[n-1];
-			uint32 nofs;
-			if (n==4)
+			uint32 modeIndex = BitBuffer.GetBits(2);
+			MPT_CHECKER_ASSUME(modeIndex < 4);
+			uint32 count = modeIndex + 2, offset;
+			if(modeIndex == 3)
 			{
-				nofs = BitBuffer.GetBits( (BitBuffer.GetBits(1)) ? nbits : 7 );
-				uint32 code;
+				offset = BitBuffer.GetBits((BitBuffer.GetBits(1)) ? modeTable[modeIndex] : 7);
+				uint32 countAdd = 7;
 				do
 				{
-					code = BitBuffer.GetBits(3);
-					n += code;
-				} while(code == 7);
+					countAdd = BitBuffer.GetBits(3);
+					count += countAdd;
+				} while(countAdd == 7);
 			} else
 			{
-				nofs = BitBuffer.GetBits(nbits);
+				offset = BitBuffer.GetBits(modeTable[modeIndex]);
 			}
-			LimitMax(n, nBytesLeft);
-			for (uint32 i=0; i<=n; i++)
+			LimitMax(count, bytesLeft);
+			for(uint32 i = 0; i < count; i++)
 			{
-				pDst[nBytesLeft-1] = (nBytesLeft+nofs < nDstLen) ? pDst[nBytesLeft+nofs] : 0;
-				if (!--nBytesLeft) break;
+				pDst[bytesLeft - 1] = (bytesLeft + offset < dstLen) ? pDst[bytesLeft + offset] : 0;
+				--bytesLeft;
 			}
 		}
 	}
@@ -112,8 +113,8 @@ static bool PP20_DoUnpack(const uint8 *pSrc, uint32 nSrcLen, uint8 *pDst, uint32
 
 struct PP20header
 {
-	char    magic[4];       // "PP20"
-	uint8be efficiency[4];
+	char  magic[4];       // "PP20"
+	uint8 efficiency[4];
 };
 
 MPT_BINARY_STRUCT(PP20header, 8)
@@ -177,31 +178,30 @@ bool UnpackPP20(std::vector<ContainerItem> &containerItems, FileReader &file, Co
 	}
 
 	containerItems.emplace_back();
-	containerItems.back().data_cache = mpt::make_unique<std::vector<char> >();
+	containerItems.back().data_cache = std::make_unique<std::vector<char> >();
 	std::vector<char> & unpackedData = *(containerItems.back().data_cache);
 
-	FileReader::off_t length = file.GetLength();
-	if(!Util::TypeCanHoldValue<uint32>(length)) return false;
+	FileReader::pos_type length = file.GetLength();
+	if(!mpt::in_range<uint32>(length)) return false;
 	// Length word must be aligned
 	if((length % 2u) != 0)
 		return false;
 
 	file.Seek(length - 4);
-	uint32 dstLen = 0;
-	dstLen |= file.ReadUint8() << 16;
-	dstLen |= file.ReadUint8() << 8;
-	dstLen |= file.ReadUint8() << 0;
-	if(dstLen == 0) return false;
+	uint32 dstLen = file.ReadUint24BE();
+	if(dstLen == 0)
+		return false;
 	try
 	{
 		unpackedData.resize(dstLen);
-	} MPT_EXCEPTION_CATCH_OUT_OF_MEMORY(e)
+	} catch(mpt::out_of_memory e)
 	{
-		MPT_EXCEPTION_DELETE_OUT_OF_MEMORY(e);
+		mpt::delete_out_of_memory(e);
 		return false;
 	}
 	file.Seek(4);
-	bool result = PP20_DoUnpack(file.GetRawData<uint8>(), static_cast<uint32>(length - 4), mpt::byte_cast<uint8 *>(unpackedData.data()), dstLen);
+	FileReader::PinnedView compressedData = file.GetPinnedView(mpt::saturate_cast<uint32>(length - 4));
+	bool result = PP20_DoUnpack(mpt::byte_cast<mpt::span<const uint8>>(compressedData.span()), mpt::byte_cast<uint8 *>(unpackedData.data()), dstLen);
 
 	if(result)
 	{
@@ -212,4 +212,7 @@ bool UnpackPP20(std::vector<ContainerItem> &containerItems, FileReader &file, Co
 }
 
 
+#endif // !MPT_WITH_ANCIENT
+
+
 OPENMPT_NAMESPACE_END
