@@ -38,17 +38,17 @@ struct DBMChunk
 	// 32-Bit chunk identifiers
 	enum ChunkIdentifiers
 	{
-		idNAME	= MagicBE("NAME"),
-		idINFO	= MagicBE("INFO"),
-		idSONG	= MagicBE("SONG"),
-		idINST	= MagicBE("INST"),
-		idVENV	= MagicBE("VENV"),
-		idPENV	= MagicBE("PENV"),
-		idPATT	= MagicBE("PATT"),
-		idPNAM	= MagicBE("PNAM"),
-		idSMPL	= MagicBE("SMPL"),
-		idDSPE	= MagicBE("DSPE"),
-		idMPEG	= MagicBE("MPEG"),
+		idNAME = MagicBE("NAME"),
+		idINFO = MagicBE("INFO"),
+		idSONG = MagicBE("SONG"),
+		idINST = MagicBE("INST"),
+		idVENV = MagicBE("VENV"),
+		idPENV = MagicBE("PENV"),
+		idPATT = MagicBE("PATT"),
+		idPNAM = MagicBE("PNAM"),
+		idSMPL = MagicBE("SMPL"),
+		idDSPE = MagicBE("DSPE"),
+		idMPEG = MagicBE("MPEG"),
 	};
 
 	uint32be id;
@@ -85,18 +85,43 @@ struct DBMInstrument
 {
 	enum DBMInstrFlags
 	{
-		smpLoop			= 0x01,
-		smpPingPongLoop	= 0x02,
+		smpLoop         = 0x01,
+		smpPingPongLoop = 0x02,
 	};
 
 	char     name[30];
-	uint16be sample;		// Sample reference
-	uint16be volume;		// 0...64
+	uint16be sample;  // Sample reference
+	uint16be volume;  // 0...64
 	uint32be sampleRate;
 	uint32be loopStart;
 	uint32be loopLength;
-	int16be  panning;		// -128...128
-	uint16be flags;			// See DBMInstrFlags
+	int16be  panning;  // -128...128
+	uint16be flags;    // See DBMInstrFlags
+
+	void ConvertToMPT(ModInstrument &mptIns) const
+	{
+		mptIns.name = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, name);
+		mptIns.nFadeOut = 0;
+		mptIns.nPan = static_cast<uint16>(panning + 128);
+		LimitMax(mptIns.nPan, uint32(256));
+		mptIns.dwFlags.set(INS_SETPANNING);
+	}
+
+	void ConvertToMPT(ModSample &mptSmp) const
+	{
+		mptSmp.Initialize(MOD_TYPE_DBM);
+		mptSmp.nVolume = std::min(static_cast<uint16>(volume), uint16(64)) * 4u;
+		mptSmp.nC5Speed = Util::muldivr(sampleRate, 8303, 8363);
+		
+		if(loopLength && (flags & (smpLoop | smpPingPongLoop)))
+		{
+			mptSmp.nLoopStart = loopStart;
+			mptSmp.nLoopEnd = mptSmp.nLoopStart + loopLength;
+			mptSmp.uFlags.set(CHN_LOOP);
+			if(flags & smpPingPongLoop)
+				mptSmp.uFlags.set(CHN_PINGPONGLOOP);
+		}
+	}
 };
 
 MPT_BINARY_STRUCT(DBMInstrument, 50)
@@ -107,26 +132,62 @@ struct DBMEnvelope
 {
 	enum DBMEnvelopeFlags
 	{
-		envEnabled	= 0x01,
-		envSustain	= 0x02,
-		envLoop		= 0x04,
+		envEnabled  = 0x01,
+		envSustain1 = 0x02,
+		envLoop     = 0x04,
+		envSustain2 = 0x08,
 	};
 
 	uint16be instrument;
-	uint8be  flags;			// See DBMEnvelopeFlags
-	uint8be  numSegments;	// Number of envelope points - 1
+	uint8be  flags;        // See DBMEnvelopeFlags
+	uint8be  numSegments;  // Number of envelope points - 1
 	uint8be  sustain1;
 	uint8be  loopBegin;
 	uint8be  loopEnd;
-	uint8be  sustain2;		// Second sustain point
+	uint8be  sustain2;  // Second sustain point
 	uint16be data[2 * 32];
+
+	void ConvertToMPT(InstrumentEnvelope &mptEnv, bool scaleEnv) const
+	{
+		if(numSegments)
+		{
+			if(flags & envEnabled) mptEnv.dwFlags.set(ENV_ENABLED);
+			if(flags & (envSustain1 | envSustain2)) mptEnv.dwFlags.set(ENV_SUSTAIN);
+			if(flags & envLoop) mptEnv.dwFlags.set(ENV_LOOP);
+		}
+
+		uint8 numPoints = std::min(numSegments.get(), uint8(31)) + 1;
+		mptEnv.resize(numPoints);
+
+		mptEnv.nLoopStart = loopBegin;
+		mptEnv.nLoopEnd = loopEnd;
+		if((flags & (envSustain1 | envSustain2)) == envSustain1)
+			mptEnv.nSustainStart = mptEnv.nSustainEnd = sustain1;
+		else if((flags & (envSustain1 | envSustain2)) == envSustain2)
+			mptEnv.nSustainStart = mptEnv.nSustainEnd = sustain2;
+		else
+			mptEnv.nSustainStart = mptEnv.nSustainEnd = std::min(sustain1, sustain2);
+
+		for(uint8 i = 0; i < numPoints; i++)
+		{
+			mptEnv[i].tick = data[i * 2];
+			uint16 val = data[i * 2 + 1];
+			if(scaleEnv)
+			{
+				// Panning envelopes are -128...128 in DigiBooster Pro 3.x
+				val = static_cast<uint16>((val + 128) / 4);
+			}
+			LimitMax(val, uint16(64));
+			mptEnv[i].value = static_cast<uint8>(val);
+		}
+	}
 };
 
 MPT_BINARY_STRUCT(DBMEnvelope, 136)
 
 
 // Note: Unlike in MOD, 1Fx, 2Fx, 5Fx / 5xF, 6Fx / 6xF and AFx / AxF are fine slides.
-static constexpr ModCommand::COMMAND dbmEffects[] =
+static constexpr EffectCommand dbmEffects[] =
 {
 	CMD_ARPEGGIO, CMD_PORTAMENTOUP, CMD_PORTAMENTODOWN, CMD_TONEPORTAMENTO,
 	CMD_VIBRATO, CMD_TONEPORTAVOL, CMD_VIBRATOVOL, CMD_TREMOLO,
@@ -137,22 +198,20 @@ static constexpr ModCommand::COMMAND dbmEffects[] =
 	CMD_NONE, CMD_PANNINGSLIDE, CMD_NONE, CMD_NONE,
 	CMD_NONE, CMD_NONE, CMD_NONE,
 #ifndef NO_PLUGINS
-	CMD_DBMECHO,	// Toggle DSP
-	CMD_MIDI,		// Wxx Echo Delay
-	CMD_MIDI,		// Xxx Echo Feedback
-	CMD_MIDI,		// Yxx Echo Mix
-	CMD_MIDI,		// Zxx Echo Cross
+	CMD_DBMECHO,  // Toggle DSP
+	CMD_MIDI,     // Wxx Echo Delay
+	CMD_MIDI,     // Xxx Echo Feedback
+	CMD_MIDI,     // Yxx Echo Mix
+	CMD_MIDI,     // Zxx Echo Cross
 #endif // NO_PLUGINS
 };
 
 
-static void ConvertDBMEffect(uint8 &command, uint8 &param)
+static std::pair<EffectCommand, uint8> ConvertDBMEffect(const uint8 cmd, uint8 param)
 {
-	uint8 oldCmd = command;
-	if(command < std::size(dbmEffects))
-		command = dbmEffects[command];
-	else
-		command = CMD_NONE;
+	EffectCommand command = CMD_NONE;
+	if(cmd < std::size(dbmEffects))
+		command = dbmEffects[cmd];
 
 	switch(command)
 	{
@@ -162,7 +221,7 @@ static void ConvertDBMEffect(uint8 &command, uint8 &param)
 		break;
 
 	case CMD_PATTERNBREAK:
-		param = ((param >> 4) * 10) + (param & 0x0F);
+		param = static_cast<uint8>(((param >> 4) * 10) + (param & 0x0F));
 		break;
 
 #ifdef MODPLUG_TRACKER
@@ -199,15 +258,15 @@ static void ConvertDBMEffect(uint8 &command, uint8 &param)
 	case CMD_MODCMDEX:
 		switch(param & 0xF0)
 		{
-		case 0x30:	// Play backwards
+		case 0x30:  // Play backwards
 			command = CMD_S3MCMDEX;
 			param = 0x9F;
 			break;
-		case 0x40:	// Turn off sound in channel (volume / portamento commands after this can't pick up the note anymore)
+		case 0x40:  // Turn off sound in channel (volume / portamento commands after this can't pick up the note anymore)
 			command = CMD_S3MCMDEX;
 			param = 0xC0;
 			break;
-		case 0x50:	// Turn on/off channel
+		case 0x50:  // Turn on/off channel
 			// TODO: Apparently this should also kill the playing note.
 			if((param & 0x0F) <= 0x01)
 			{
@@ -215,7 +274,7 @@ static void ConvertDBMEffect(uint8 &command, uint8 &param)
 				param = (param == 0x50) ? 0x00 : 0x40;
 			}
 			break;
-		case 0x70:	// Coarse offset
+		case 0x70:  // Coarse offset
 			command = CMD_S3MCMDEX;
 			param = 0xA0 | (param & 0x0F);
 			break;
@@ -230,7 +289,7 @@ static void ConvertDBMEffect(uint8 &command, uint8 &param)
 		break;
 
 	case CMD_KEYOFF:
-		if (param == 0)
+		if(param == 0)
 		{
 			// TODO key off at tick 0
 		}
@@ -238,8 +297,13 @@ static void ConvertDBMEffect(uint8 &command, uint8 &param)
 
 	case CMD_MIDI:
 		// Encode echo parameters into fixed MIDI macros
-		param = 128 + (oldCmd - 32) * 32 + param / 8;
+		param = static_cast<uint8>(128 + (cmd - 32) * 32 + param / 8);
+		break;
+
+	default:
+		break;
 	}
+	return {command, param};
 }
 
 
@@ -255,35 +319,7 @@ static void ReadDBMEnvelopeChunk(FileReader chunk, EnvelopeType envType, CSoundF
 		uint16 dbmIns = dbmEnv.instrument;
 		if(dbmIns > 0 && dbmIns <= sndFile.GetNumInstruments() && (sndFile.Instruments[dbmIns] != nullptr))
 		{
-			ModInstrument *mptIns = sndFile.Instruments[dbmIns];
-			InstrumentEnvelope &mptEnv = mptIns->GetEnvelope(envType);
-
-			if(dbmEnv.numSegments)
-			{
-				if(dbmEnv.flags & DBMEnvelope::envEnabled) mptEnv.dwFlags.set(ENV_ENABLED);
-				if(dbmEnv.flags & DBMEnvelope::envSustain) mptEnv.dwFlags.set(ENV_SUSTAIN);
-				if(dbmEnv.flags & DBMEnvelope::envLoop) mptEnv.dwFlags.set(ENV_LOOP);
-			}
-
-			uint8 numPoints = std::min(dbmEnv.numSegments.get(), uint8(31)) + 1;
-			mptEnv.resize(numPoints);
-
-			mptEnv.nLoopStart = dbmEnv.loopBegin;
-			mptEnv.nLoopEnd = dbmEnv.loopEnd;
-			mptEnv.nSustainStart = mptEnv.nSustainEnd = dbmEnv.sustain1;
-
-			for(uint8 i = 0; i < numPoints; i++)
-			{
-				mptEnv[i].tick = dbmEnv.data[i * 2];
-				uint16 val = dbmEnv.data[i * 2 + 1];
-				if(scaleEnv)
-				{
-					// Panning envelopes are -128...128 in DigiBooster Pro 3.x
-					val = (val + 128) / 4;
-				}
-				LimitMax(val, uint16(64));
-				mptEnv[i].value = static_cast<uint8>(val);
-			}
+			dbmEnv.ConvertToMPT(sndFile.Instruments[dbmIns]->GetEnvelope(envType), scaleEnv);
 		}
 	}
 }
@@ -344,18 +380,18 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 		return false;
 	}
 
-	InitializeGlobals(MOD_TYPE_DBM);
-	InitializeChannels();
+	InitializeGlobals(MOD_TYPE_DBM, Clamp<uint16, uint16>(infoData.channels, 1, MAX_BASECHANNELS));  // Note: MAX_BASECHANNELS is currently 192, but DBPro 3 apparently supports up to 254 channels.
 	m_SongFlags = SONG_ITCOMPATGXX | SONG_ITOLDEFFECTS;
-	m_nChannels = Clamp<uint16, uint16>(infoData.channels, 1, MAX_BASECHANNELS);	// note: MAX_BASECHANNELS is currently 127, but DBPro 2 supports up to 128 channels, DBPro 3 apparently up to 254.
 	m_nInstruments = std::min(static_cast<INSTRUMENTINDEX>(infoData.instruments), static_cast<INSTRUMENTINDEX>(MAX_INSTRUMENTS - 1));
 	m_nSamples = std::min(static_cast<SAMPLEINDEX>(infoData.samples), static_cast<SAMPLEINDEX>(MAX_SAMPLES - 1));
 	m_playBehaviour.set(kSlidesAtSpeed1);
 	m_playBehaviour.reset(kITVibratoTremoloPanbrello);
 	m_playBehaviour.reset(kITArpeggio);
+	m_playBehaviour.reset(kITInstrWithNoteOff);
+	m_playBehaviour.reset(kITInstrWithNoteOffOldEffects);
 
-	m_modFormat.formatName = U_("DigiBooster Pro");
-	m_modFormat.type = U_("dbm");
+	m_modFormat.formatName = UL_("DigiBooster Pro");
+	m_modFormat.type = UL_("dbm");
 	m_modFormat.madeWithTracker = MPT_UFORMAT("DigiBooster Pro {}.{}")(mpt::ufmt::hex(fileHeader.trkVerHi), mpt::ufmt::hex(fileHeader.trkVerLo));
 	m_modFormat.charset = mpt::Charset::Amiga_no_C1;
 
@@ -404,41 +440,41 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 #endif // MPT_DBM_USE_REAL_SUBSONGS
 
 	// Read instruments
+	std::map<SAMPLEINDEX, SAMPLEINDEX> copySample;
 	if(FileReader instChunk = chunks.GetChunk(DBMChunk::idINST))
 	{
+		std::set<SAMPLEINDEX> sampleUsed;
 		for(INSTRUMENTINDEX i = 1; i <= GetNumInstruments(); i++)
 		{
 			DBMInstrument instrHeader;
 			instChunk.ReadStruct(instrHeader);
 
-			ModInstrument *mptIns = AllocateInstrument(i, instrHeader.sample);
-			if(mptIns == nullptr || instrHeader.sample >= MAX_SAMPLES)
+			SAMPLEINDEX mappedSample = instrHeader.sample;
+			if(sampleUsed.count(mappedSample) && CanAddMoreSamples())
 			{
+				ModSample mptSmp;
+				instrHeader.ConvertToMPT(mptSmp);
+				const ModSample &origSmp = Samples[mappedSample];
+				if(mptSmp.nVolume != origSmp.nVolume
+					|| mptSmp.uFlags != origSmp.uFlags
+					|| mptSmp.nLoopStart != origSmp.nLoopStart
+					|| mptSmp.nLoopEnd != origSmp.nLoopEnd
+					|| mptSmp.nC5Speed != origSmp.nC5Speed)
+				{
+					// Need to duplicate
+					mappedSample = ++m_nSamples;
+					copySample.emplace(mappedSample, instrHeader.sample);
+				}
+			}
+			ModInstrument *mptIns = AllocateInstrument(i, mappedSample);
+			if(mptIns == nullptr || mappedSample >= MAX_SAMPLES)
 				continue;
-			}
-
-			mptIns->name = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, instrHeader.name);
-			m_szNames[instrHeader.sample] = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, instrHeader.name);
-
-			mptIns->nFadeOut = 0;
-			mptIns->nPan = static_cast<uint16>(instrHeader.panning + 128);
-			LimitMax(mptIns->nPan, uint32(256));
-			mptIns->dwFlags.set(INS_SETPANNING);
 
+			instrHeader.ConvertToMPT(*mptIns);
 			// Sample Info
-			ModSample &mptSmp = Samples[instrHeader.sample];
-			mptSmp.Initialize();
-			mptSmp.nVolume = std::min(static_cast<uint16>(instrHeader.volume), uint16(64)) * 4u;
-			mptSmp.nC5Speed = Util::muldivr(instrHeader.sampleRate, 8303, 8363);
-
-			if(instrHeader.loopLength && (instrHeader.flags & (DBMInstrument::smpLoop | DBMInstrument::smpPingPongLoop)))
-			{
-				mptSmp.nLoopStart = instrHeader.loopStart;
-				mptSmp.nLoopEnd = mptSmp.nLoopStart + instrHeader.loopLength;
-				mptSmp.uFlags.set(CHN_LOOP);
-				if(instrHeader.flags & DBMInstrument::smpPingPongLoop)
-					mptSmp.uFlags.set(CHN_PINGPONGLOOP);
-			}
+			instrHeader.ConvertToMPT(Samples[mappedSample]);
+			m_szNames[mappedSample] = mptIns->name;
+			sampleUsed.insert(mappedSample);
 		}
 
 		// Read envelopes
@@ -463,7 +499,7 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 	if(patternChunk.IsValid() && (loadFlags & loadPatternData))
 	{
 		FileReader patternNameChunk = chunks.GetChunk(DBMChunk::idPNAM);
-		patternNameChunk.Skip(1);	// Encoding, should be UTF-8 or ASCII
+		patternNameChunk.Skip(1);  // Encoding (0 = unspecified ASCII-compatible 8-bit encoding, 106 = UTF-8)
 
 		Patterns.ResizeArray(infoData.patterns);
 		std::vector<std::pair<EffectCommand, ModCommand::PARAM>> lostGlobalCommands;
@@ -480,7 +516,7 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 			patternNameChunk.ReadSizedString<uint8be, mpt::String::maybeNullTerminated>(patName);
 			Patterns[pat].SetName(patName);
 
-			PatternRow patRow = Patterns[pat].GetRow(0);
+			auto patRow = Patterns[pat].GetRow(0);
 			ROWINDEX row = 0;
 			lostGlobalCommands.clear();
 			while(chunk.CanRead(1))
@@ -503,7 +539,7 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 					continue;
 				}
 
-				ModCommand dummy = ModCommand::Empty();
+				ModCommand dummy{};
 				ModCommand &m = ch <= GetNumChannels() ? patRow[ch - 1] : dummy;
 
 				const uint8 b = chunk.ReadUint8();
@@ -515,7 +551,7 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 					if(note == 0x1F)
 						m.note = NOTE_KEYOFF;
 					else if(note > 0 && note < 0xFE)
-						m.note = ((note >> 4) * 12) + (note & 0x0F) + 13;
+						m.note = static_cast<ModCommand::NOTE>(((note >> 4) * 12) + (note & 0x0F) + 13);
 				}
 				if(b & 0x02)
 				{
@@ -523,28 +559,32 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 				}
 				if(b & 0x3C)
 				{
-					uint8 cmd1 = 0, cmd2 = 0, param1 = 0, param2 = 0;
-					if(b & 0x04) cmd2 = chunk.ReadUint8();
-					if(b & 0x08) param2 = chunk.ReadUint8();
-					if(b & 0x10) cmd1 = chunk.ReadUint8();
-					if(b & 0x20) param1 = chunk.ReadUint8();
-					ConvertDBMEffect(cmd1, param1);
-					ConvertDBMEffect(cmd2, param2);
+					uint8 c1 = 0, p1 = 0, c2 = 0, p2 = 0;
+					if(b & 0x04) c2 = chunk.ReadUint8();
+					if(b & 0x08) p2 = chunk.ReadUint8();
+					if(b & 0x10) c1 = chunk.ReadUint8();
+					if(b & 0x20) p1 = chunk.ReadUint8();
+					auto [cmd1, param1] = ConvertDBMEffect(c1, p1);
+					auto [cmd2, param2] = ConvertDBMEffect(c2, p2);
 
-					if (cmd2 == CMD_VOLUME || (cmd2 == CMD_NONE && cmd1 != CMD_VOLUME))
+					if(cmd2 == CMD_VOLUME || (cmd2 == CMD_NONE && cmd1 != CMD_VOLUME))
 					{
 						std::swap(cmd1, cmd2);
 						std::swap(param1, param2);
+					} else if(cmd1 == CMD_TONEPORTAMENTO && cmd2 == CMD_OFFSET && param2 == 0)
+					{
+						// Offset + Portamento: Ignore portamento. If the offset command has a non-zero parameter, keep it for effect memory.
+						cmd2 = CMD_NONE;
+					} else if(cmd2 == CMD_TONEPORTAMENTO && cmd1 == CMD_OFFSET && param1 == 0)
+					{
+						// Ditto
+						cmd1 = CMD_NONE;
 					}
 
-					const auto lostCommand = ModCommand::TwoRegularCommandsToMPT(cmd1, param1, cmd2, param2);
+					const auto lostCommand = m.FillInTwoCommands(cmd1, param1, cmd2, param2, true);
 					if(ModCommand::IsGlobalCommand(lostCommand.first, lostCommand.second))
 						lostGlobalCommands.insert(lostGlobalCommands.begin(), lostCommand);  // Insert at front so that the last command of same type "wins"
 
-					m.volcmd = cmd1;
-					m.vol = param1;
-					m.command = cmd2;
-					m.param = param2;
 #ifdef MODPLUG_TRACKER
 					m.ExtendedMODtoS3MEffect();
 #endif // MODPLUG_TRACKER
@@ -566,7 +606,7 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 		if(hasEchoEnable)
 		{
 			// If there are any Vxx effects to dynamically enable / disable echo, use the CHN_NOFX flag.
-			for(CHANNELINDEX i = 0; i < m_nChannels; i++)
+			for(CHANNELINDEX i = 0; i < GetNumChannels(); i++)
 			{
 				ChnSettings[i].nMixPlugin = 1;
 				ChnSettings[i].dwFlags.set(CHN_NOFX);
@@ -583,7 +623,7 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 			for(uint16 i = 0; i < maskLen; i++)
 			{
 				bool enabled = (dspChunk.ReadUint8() == 0);
-				if(i < m_nChannels)
+				if(i < GetNumChannels())
 				{
 					if(hasEchoEnable)
 					{
@@ -642,6 +682,13 @@ bool CSoundFile::ReadDBM(FileReader &file, ModLoadingFlags loadFlags)
 	{
 		for(SAMPLEINDEX smp = 1; smp <= GetNumSamples(); smp++)
 		{
+			if(auto copyFrom = copySample.find(smp); copyFrom != copySample.end())
+			{
+				Samples[smp].nLength = Samples[copyFrom->second].nLength;
+				Samples[smp].CopyWaveform(Samples[copyFrom->second]);
+				continue;
+			}
+
 			uint32 sampleFlags = sampleChunk.ReadUint32BE();
 			uint32 sampleLength = sampleChunk.ReadUint32BE();
 
