@@ -11,18 +11,20 @@
 #include "stdafx.h"
 #include "Loaders.h"
 #include "BitReader.h"
+#include "mpt/endian/int24.hpp"
 
 OPENMPT_NAMESPACE_BEGIN
 
 struct DSymFileHeader
 {
+	using uint24le = mpt::uint24le;
+
 	char     magic[8];
 	uint8le  version;      // 0 / 1
 	uint8le  numChannels;  // 1...8
 	uint16le numOrders;    // 0...4096
 	uint16le numTracks;    // 0...4096
-	uint16le infoLenLo;
-	uint8le  infoLenHi;
+	uint24le infoLen;
 
 	bool Validate() const
 	{
@@ -112,7 +114,7 @@ static std::vector<std::byte> DecompressDSymLZW(FileReader &file, uint32 size)
 
 			dictionary[nextIndex].value = match[writeOffset];
 			dictionary[nextIndex].prev = prevCode;
-			
+
 			nextIndex++;
 			if(nextIndex != MaxNodes && nextIndex == (1u << codeSize))
 				codeSize++;
@@ -123,7 +125,9 @@ static std::vector<std::byte> DecompressDSymLZW(FileReader &file, uint32 size)
 	MPT_ASSERT(output.size() == size);
 
 	// Align length to 4 bytes
-	file.Seek(startPos + ((bitFile.GetPosition() - startPos + 3u) & ~FileReader::off_t(3)));
+	file.Seek(startPos + ((bitFile.GetPosition() - startPos + 3u) & ~FileReader::pos_type(3)));
+	// cppcheck false-positive
+	// cppcheck-suppress returnDanglingLifetime
 	return output;
 }
 
@@ -181,7 +185,7 @@ static std::vector<std::byte> DecompressDSymSigmaDelta(FileReader &file, uint32
 	}
 
 	// Align length to 4 bytes
-	file.Seek(startPos + ((bitFile.GetPosition() - startPos + 3u) & ~FileReader::off_t(3)));
+	file.Seek(startPos + ((bitFile.GetPosition() - startPos + 3u) & ~FileReader::pos_type(3)));
 	return output;
 }
 
@@ -228,20 +232,18 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 	file.Rewind();
 	if(!file.ReadStruct(fileHeader) || !fileHeader.Validate())
 		return false;
-	if(!file.CanRead(mpt::saturate_cast<FileReader::off_t>(fileHeader.GetHeaderMinimumAdditionalSize())))
+	if(!file.CanRead(mpt::saturate_cast<FileReader::pos_type>(fileHeader.GetHeaderMinimumAdditionalSize())))
 		return false;
 	if(loadFlags == onlyVerifyHeader)
 		return true;
 
-	InitializeGlobals(MOD_TYPE_MOD);
+	InitializeGlobals(MOD_TYPE_MOD, fileHeader.numChannels);
 	m_SongFlags.set(SONG_IMPORTED | SONG_AMIGALIMITS);
 	m_SongFlags.reset(SONG_ISAMIGA);
-	m_nChannels = fileHeader.numChannels;
 	m_nSamples = 63;
 
-	for(CHANNELINDEX chn = 0; chn < m_nChannels; chn++)
+	for(CHANNELINDEX chn = 0; chn < GetNumChannels(); chn++)
 	{
-		InitChannel(chn);
 		ChnSettings[chn].nPan = (((chn & 3) == 1) || ((chn & 3) == 2)) ? 64 : 192;
 	}
 
@@ -289,14 +291,14 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 		if(!(loadFlags & loadPatternData) || !Patterns.Insert(pat, 64))
 			continue;
 
-		for(CHANNELINDEX chn = 0; chn < m_nChannels; chn++)
+		for(CHANNELINDEX chn = 0; chn < GetNumChannels(); chn++)
 		{
-			const uint16 track = sequence[pat * m_nChannels + chn];
+			const uint16 track = sequence[pat * GetNumChannels() + chn];
 			if(track >= fileHeader.numTracks)
 				continue;
 
 			ModCommand *m = Patterns[pat].GetpModCommand(0, chn);
-			for(ROWINDEX row = 0; row < 64; row++, m += m_nChannels)
+			for(ROWINDEX row = 0; row < 64; row++, m += GetNumChannels())
 			{
 				const auto data = tracks.subspan(track * 256 + row * 4, 4);
 				m->note = data[0] & 0x3F;
@@ -314,10 +316,9 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 				if(command == 0 && param == 0)
 					continue;
 
-				m->command = command;
 				m->param = static_cast<uint8>(param);
 				m->vol = static_cast<ModCommand::VOL>(param >> 8);
-				
+
 				switch(command)
 				{
 					case 0x00:  // 00 xyz Normal play or Arpeggio + Volume Slide Up
@@ -326,8 +327,7 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 					case 0x20:  // 20 xyz Normal play or Arpeggio + Volume Slide Down
 					case 0x21:  // 21 xyy Slide Up + Volume Slide Down
 					case 0x22:  // 22 xyy Slide Down + Volume Slide Down
-						m->command &= 0x0F;
-						ConvertModCommand(*m);
+						ConvertModCommand(*m, command & 0x0F, m->param);
 						if(m->vol)
 							m->volcmd = (command < 0x20) ? VOLCMD_VOLSLIDEUP : VOLCMD_VOLSLIDEDOWN;
 						break;
@@ -337,7 +337,7 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 					case 0x06:  // 06 xyz Vibrato + Volume Slide
 					case 0x07:  // 07 xyz Tremolo
 					case 0x0C:  // 0C xyy Set Volume
-						ConvertModCommand(*m);
+						ConvertModCommand(*m, command, m->param);
 						break;
 					case 0x09:  // 09 xxx Set Sample Offset
 						m->command = CMD_OFFSET;
@@ -352,8 +352,7 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 					case 0x2A:  // 2A xyz Volume Slide + Fine Slide Down
 						if(param < 0xFF)
 						{
-							m->command &= 0x0F;
-							ConvertModCommand(*m);
+							ConvertModCommand(*m, command & 0x0F, m->param);
 						} else
 						{
 							m->command = CMD_MODCMDEX;
@@ -426,7 +425,7 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 						break;
 					case 0x2B:  // 2B xyy Line Jump
 						m->command = CMD_PATTERNBREAK;
-						for(CHANNELINDEX brkChn = 0; brkChn < m_nChannels; brkChn++)
+						for(CHANNELINDEX brkChn = 0; brkChn < GetNumChannels(); brkChn++)
 						{
 							ModCommand &cmd = *(m - chn + brkChn);
 							if(cmd.command != CMD_NONE)
@@ -594,7 +593,7 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 		}
 	}
 
-	if(const uint32 infoLen = fileHeader.infoLenLo | (fileHeader.infoLenHi << 16); infoLen > 0)
+	if(const uint32 infoLen = fileHeader.infoLen.get(); infoLen > 0)
 	{
 		std::vector<std::byte> infoData;
 		if(!ReadDSymChunk(file, infoData, infoLen))
@@ -604,8 +603,8 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 	}
 
 	m_modFormat.formatName = MPT_UFORMAT("Digital Symphony v{}")(fileHeader.version);
-	m_modFormat.type = U_("dsym");  // RISC OS doesn't use file extensions but this is a common abbreviation used for this tracker
-	m_modFormat.madeWithTracker = U_("Digital Symphony");
+	m_modFormat.type = UL_("dsym");  // RISC OS doesn't use file extensions but this is a common abbreviation used for this tracker
+	m_modFormat.madeWithTracker = UL_("Digital Symphony");
 	m_modFormat.charset = mpt::Charset::RISC_OS;
 
 	return true;
@@ -613,3 +612,4 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 
 
 OPENMPT_NAMESPACE_END
+
