@@ -35,14 +35,20 @@ struct SymFileHeader
 {
 	char     magic[4];  // "SymM"
 	uint32be version;
+	// Technically this is already the first chunk; for simplicity we always assume that the channel count comes first (which in practice it does)
+	int32be  firstChunkID;
+	uint32be numChannels;
 
 	bool Validate() const
 	{
-		return !std::memcmp(magic, "SymM", 4) && version == 1;
+		return !std::memcmp(magic, "SymM", 4)
+			&& version == 1
+			&& firstChunkID == -1
+			&& numChannels > 0 && numChannels <= 256;
 	}
 };
 
-MPT_BINARY_STRUCT(SymFileHeader, 8)
+MPT_BINARY_STRUCT(SymFileHeader, 16)
 
 
 struct SymEvent
@@ -276,8 +282,8 @@ struct SymTranswaveInst
 		std::pair<SmpLength, SmpLength> ConvertLoop(const ModSample &mptSmp) const
 		{
 			const double loopScale = static_cast<double>(mptSmp.nLength) / (100 << 16);
-			const SmpLength start  = mpt::saturate_cast<SmpLength>(loopScale * std::min(uint32(100 << 16), loopStart.get()));
-			const SmpLength length = mpt::saturate_cast<SmpLength>(loopScale * std::min(uint32(100 << 16), loopLen.get()));
+			const SmpLength start  = mpt::saturate_trunc<SmpLength>(loopScale * std::min(uint32(100 << 16), loopStart.get()));
+			const SmpLength length = mpt::saturate_trunc<SmpLength>(loopScale * std::min(uint32(100 << 16), loopLen.get()));
 			return {start, std::min(mptSmp.nLength - start, length)};
 		}
 	};
@@ -503,7 +509,7 @@ struct SymInstrument
 				return;
 			CopySample<SC::ConversionChain<SC::Convert<int16, int8>, SC::DecodeIdentity<int8>>>(newSample, mptSmp.nLength * mptSmp.GetNumChannels(), 1, mptSmp.sample8(), mptSmp.GetSampleSizeInBytes(), 1);
 			mptSmp.uFlags.set(CHN_16BIT);
-			ctrlSmp::ReplaceSample(mptSmp, newSample, mptSmp.nLength, sndFile);
+			mptSmp.ReplaceWaveform(newSample, mptSmp.nLength, sndFile);
 		}
 
 		// Highpass
@@ -640,7 +646,6 @@ struct SymInstrument
 			if(!newSample)
 				return;
 
-			mptSmp.nLength = newLength;
 			std::memcpy(newSample, mptSmp.sampleb(), (loopStart + loopLen) * bps);
 			for(uint8 i = 0; i < numRepetitions; i++)
 			{
@@ -648,7 +653,7 @@ struct SymInstrument
 			}
 			std::memcpy(newSample + loopEnd * bps, mptSmp.sampleb() + (loopStart + loopLen) * bps, (newLength - loopEnd) * bps);
 			
-			ctrlSmp::ReplaceSample(mptSmp, newSample, mptSmp.nLength, sndFile);
+			mptSmp.ReplaceWaveform(newSample, newLength, sndFile);
 		}
 	}
 
@@ -657,16 +662,16 @@ struct SymInstrument
 		if(type != Loop && type != Sustain)
 			return {0, 0};
 
-		SmpLength loopStart = static_cast<SmpLength>(std::min(loopStartHigh.get(), uint8(100)));
-		SmpLength loopLen = static_cast<SmpLength>(std::min(loopLenHigh.get(), uint8(100)));
+		SmpLength loopStart = std::min(loopStartHigh.get(), uint8(100));
+		SmpLength loopLen = std::min(loopLenHigh.get(), uint8(100));
 		if(sampleFlags & NewLoopSystem)
 		{
 			loopStart = (loopStart << 16) + loopStartFine;
 			loopLen = (loopLen << 16) + loopLenFine;
 
 			const double loopScale = static_cast<double>(mptSmp.nLength) / (100 << 16);
-			loopStart = mpt::saturate_cast<SmpLength>(loopStart * loopScale);
-			loopLen = std::min(mptSmp.nLength - loopStart, mpt::saturate_cast<SmpLength>(loopLen * loopScale));
+			loopStart = std::min(mptSmp.nLength, mpt::saturate_trunc<SmpLength>(loopStart * loopScale));
+			loopLen = std::min(mptSmp.nLength - loopStart, mpt::saturate_trunc<SmpLength>(loopLen * loopScale));
 		} else if(mptSmp.HasSampleData())
 		{
 			// The order of operations here may seem weird as it reduces precision, but it's taken directly from the original assembly source (UpdateRecalcLoop)
@@ -846,7 +851,7 @@ static std::vector<std::byte> DecodeSymChunk(FileReader &file)
 		// When using a fuzzer, we should not care if the decompressed buffer has the correct size.
 		// This makes finding new interesting test cases much easier.
 		if(remain)
-			std::vector<std::byte>{}.swap(data);
+			mpt::reconstruct(data);
 #endif
 	} else
 	{
@@ -952,7 +957,7 @@ static bool ConvertDSP(const SymEvent event, MIDIMacroConfigData::Macro &macro,
 	{
 		// Symphonie practically uses the same filter for this as for the sample processing.
 		// The cutoff and resonance are an approximation.
-		const uint8 type = event.note % 5u;
+		const uint8 type = static_cast<uint8>(event.note % 5u);
 		const uint8 cutoff = sndFile.FrequencyToCutOff(event.param * 10000.0 / 240.0);
 		const uint8 reso = static_cast<uint8>(std::min(127, event.inst * 127 / 185));
 
@@ -980,6 +985,30 @@ static bool ConvertDSP(const SymEvent event, MIDIMacroConfigData::Macro &macro,
 }
 
 
+static uint8 MapToClosestMidiMacro(const SymEvent event, std::map<SymEvent, uint8> &macroMap)
+{
+	if(event.command == SymEvent::DSPDelay)
+		return 0;
+	uint8 bestMatch = 0;
+	uint32 bestDistance = uint32_max;
+	for(const auto &m : macroMap)
+	{
+		const auto &mapEvent = m.first;
+		if(event.command != mapEvent.command || event.note != mapEvent.note)
+			continue;
+		const uint32 diff1 = static_cast<uint32>(event.param) - mapEvent.param, diff2 = static_cast<uint32>(event.inst) - mapEvent.inst;
+		const uint32 distance = diff1 * diff1 + diff2 * diff2;
+		if(distance >= bestDistance)
+			continue;
+
+		bestMatch = m.second;
+		bestDistance = distance;
+	}
+	macroMap[event] = bestMatch;
+	return bestMatch;
+}
+
+
 CSoundFile::ProbeResult CSoundFile::ProbeFileHeaderSymMOD(MemoryFileReader file, const uint64 *pfilesize)
 {
 	MPT_UNREFERENCED_PARAMETER(pfilesize);
@@ -988,10 +1017,6 @@ CSoundFile::ProbeResult CSoundFile::ProbeFileHeaderSymMOD(MemoryFileReader file,
 		return ProbeWantMoreData;
 	if(!fileHeader.Validate())
 		return ProbeFailure;
-	if(!file.CanRead(sizeof(uint32be)))
-		return ProbeWantMoreData;
-	if(file.ReadInt32BE() >= 0)
-		return ProbeFailure;
 	return ProbeSuccess;
 }
 
@@ -1002,16 +1027,15 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 	SymFileHeader fileHeader;
 	if(!file.ReadStruct(fileHeader) || !fileHeader.Validate())
 		return false;
-	if(file.ReadInt32BE() >= 0)
-		return false;
 	else if(loadFlags == onlyVerifyHeader)
 		return true;
 
-	InitializeGlobals(MOD_TYPE_MPT);
+	InitializeGlobals(MOD_TYPE_MPT, std::min(MAX_BASECHANNELS, static_cast<CHANNELINDEX>(fileHeader.numChannels)));
 
-	m_SongFlags.set(SONG_LINEARSLIDES | SONG_EXFILTERRANGE | SONG_IMPORTED);
+	m_SongFlags.set(SONG_LINEARSLIDES | SONG_EXFILTERRANGE | SONG_AUTO_VIBRATO | SONG_AUTO_TREMOLO | SONG_IMPORTED);
 	m_playBehaviour = GetDefaultPlaybackBehaviour(MOD_TYPE_IT);
 	m_playBehaviour.reset(kITShortSampleRetrig);
+	m_nSamplePreAmp = Clamp(512 / GetNumChannels(), 16, 128);
 
 	enum class ChunkType : int32
 	{
@@ -1044,12 +1068,12 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 	uint16 sampleBoost   = 2500;
 	bool isSymphoniePro  = false;
 	bool externalSamples = false;
+	bool unknownHunks    = false;
 	std::vector<SymPosition> positions;
 	std::vector<SymSequence> sequences;
 	std::vector<SymEvent> patternData;
 	std::vector<SymInstrument> instruments;
 
-	file.SkipBack(sizeof(int32));
 	while(file.CanRead(sizeof(int32)))
 	{
 		const ChunkType chunkType = static_cast<ChunkType>(file.ReadInt32BE());
@@ -1057,11 +1081,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 		{
 		// Simple values
 		case ChunkType::NumChannels:
-			if(auto numChannels = static_cast<CHANNELINDEX>(file.ReadUint32BE()); !m_nChannels && numChannels > 0 && numChannels <= MAX_BASECHANNELS)
-			{
-				m_nChannels = numChannels;
-				m_nSamplePreAmp = Clamp(512 / m_nChannels, 16, 128);
-			}
+			file.Skip(sizeof(uint32be));  // Already handled
 			break;
 
 		case ChunkType::TrackLength:
@@ -1076,7 +1096,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 			break;
 
 		case ChunkType::Tempo:
-			m_nDefaultTempo = TEMPO(1.24 * std::min(file.ReadUint32BE(), uint32(800)));
+			Order().SetDefaultTempo(TEMPO(1.24 * std::min(file.ReadUint32BE(), uint32(800))));
 			break;
 
 		// Unused values
@@ -1202,16 +1222,19 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 			file.Skip(file.ReadUint32BE());
 			break;
 
-		// Unrecognized chunk/value type
+		// Unrecognized chunk/value type (e.g. garbage at the end of Natsh1.SymMOD)
 		default:
-			return false;
+			unknownHunks = true;
+			break;
 		}
 	}
 
-	if(!m_nChannels || !trackLen || instruments.empty())
+	if(!trackLen || instruments.empty())
 		return false;
 	if((loadFlags & loadPatternData) && (positions.empty() || patternData.empty() || sequences.empty()))
 		return false;
+	if(unknownHunks)
+		AddToLog(LogWarning, U_("Unknown hunks were found and ignored."));
 
 	// Let's hope noone is going to use the 256th instrument ;)
 	if(instruments.size() >= MAX_INSTRUMENTS)
@@ -1233,7 +1256,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 			
 			auto filename = mpt::PathString::FromUnicode(mpt::ToUnicode(mpt::Charset::Amiga_no_C1, symInst.GetName()));
 			if(file.GetOptionalFileName())
-				filename = file.GetOptionalFileName()->GetPath() + filename.GetFullFileName();
+				filename = file.GetOptionalFileName()->GetDirectoryWithDrive() + filename.GetFilename();
 			
 			if(!LoadExternalSample(sample, filename))
 				AddToLog(LogError, MPT_UFORMAT("Unable to load sample {}: {}")(sample, filename));
@@ -1312,7 +1335,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 	std::map<SymEvent, uint8> macroMap;
 
 	bool useDSP = false;
-	const uint32 patternSize = m_nChannels * trackLen;
+	const uint32 patternSize = fileHeader.numChannels * trackLen;
 	const PATTERNINDEX numPatterns = mpt::saturate_cast<PATTERNINDEX>(patternData.size() / patternSize);
 
 	Patterns.ResizeArray(numPatterns);
@@ -1331,7 +1354,9 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 		uint8 channelVol       = 100;    // Volume multiplier, 0...100
 		uint8 calculatedVol    = 64;     // Final channel volume
 		uint8 fromAdd          = 0;      // Base sample offset for FROM and FR&P effects
+		bool  retrigVibrato    = false;
 		uint8 curVibrato       = 0;
+		bool  retrigTremolo    = false;
 		uint8 curTremolo       = 0;
 		uint8 sampleVibSpeed   = 0;
 		uint8 sampleVibDepth   = 0;
@@ -1340,7 +1365,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 		uint16 retriggerRemain = 0;
 		uint16 tonePortaRemain = 0;
 	};
-	std::vector<ChnState> chnStates(m_nChannels);
+	std::vector<ChnState> chnStates(GetNumChannels());
 
 	// In Symphonie, sequences represent the structure of a song, and not separate songs like in OpenMPT. Hence they will all be loaded into the same ModSequence.
 	for(SymSequence &seq : sequences)
@@ -1360,7 +1385,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 		// Sequences are all part of the same song, just add a skip index as a divider
 		ModSequence &order = Order();
 		if(!order.empty())
-			order.push_back(ModSequence::GetIgnoreIndex());
+			order.push_back(PATTERNINDEX_SKIP);
 
 		for(auto &pos : seqPositions)
 		{
@@ -1387,19 +1412,21 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 				uint8 patternSpeed = static_cast<uint8>(pos.speed);
 
 				// This may intentionally read into the next pattern
-				auto srcEvent = patternData.cbegin() + (pos.pattern * patternSize) + (pos.start * m_nChannels);
+				auto srcEvent = patternData.cbegin() + (pos.pattern * patternSize) + (pos.start * fileHeader.numChannels);
 				const SymEvent emptyEvent{};
 				ModCommand syncPlayCommand;
 				for(ROWINDEX row = 0; row < pos.length; row++)
 				{
 					ModCommand *rowBase = Patterns[patternIndex].GetpModCommand(row, 0);
 					bool applySyncPlay = false;
-					for(CHANNELINDEX chn = 0; chn < m_nChannels; chn++)
+					for(CHANNELINDEX chn = 0; chn < fileHeader.numChannels; chn++)
 					{
-						ModCommand &m = rowBase[chn];
 						const SymEvent &event = (srcEvent != patternData.cend()) ? *srcEvent : emptyEvent;
 						if(srcEvent != patternData.cend())
 							srcEvent++;
+						if(chn >= GetNumChannels())
+							continue;
+						ModCommand &m = rowBase[chn];
 
 						int8 note = (event.note >= 0 && event.note <= 84) ? event.note + 25 : -1;
 						uint8 origInst = event.inst;
@@ -1426,8 +1453,8 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 							m = syncPlayCommand;
 							if(m.command == CMD_NONE && chnState.calculatedVol != chnStates[chn - 1].calculatedVol)
 							{
-								m.command = CMD_CHANNELVOLUME;
-								m.param = chnState.calculatedVol = chnStates[chn - 1].calculatedVol;
+								chnState.calculatedVol = chnStates[chn - 1].calculatedVol;
+								m.SetEffectCommand(CMD_CHANNELVOLUME, chnState.calculatedVol);
 							}
 							if(!event.IsGlobal())
 								continue;
@@ -1442,37 +1469,32 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 								switch(event.param)
 								{
 								case SymEvent::StopSample:
-									m.volcmd = VOLCMD_PLAYCONTROL;
-									m.vol = 0;
+									m.SetVolumeCommand(VOLCMD_PLAYCONTROL, 0);
 									chnState.stopped = true;
 									break;
 
 								case SymEvent::ContSample:
-									m.volcmd = VOLCMD_PLAYCONTROL;
-									m.vol = 1;
+									m.SetVolumeCommand(VOLCMD_PLAYCONTROL, 1);
 									chnState.stopped = false;
 									break;
 
 								case SymEvent::KeyOff:
 									if(m.note == NOTE_NONE)
 										m.note = chnState.lastNote;
-									m.volcmd = VOLCMD_OFFSET;
-									m.vol = 1;
+									m.SetVolumeCommand(VOLCMD_OFFSET, 1);
 									break;
 
 								case SymEvent::SpeedDown:
 									if(patternSpeed > 1)
 									{
-										m.command = CMD_SPEED;
-										m.param = --patternSpeed;
+										m.SetEffectCommand(CMD_SPEED, --patternSpeed);
 									}
 									break;
 
 								case SymEvent::SpeedUp:
 									if(patternSpeed < 0xFF)
 									{
-										m.command = CMD_SPEED;
-										m.param = ++patternSpeed;
+										m.SetEffectCommand(CMD_SPEED, ++patternSpeed);
 									}
 									break;
 
@@ -1481,36 +1503,30 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 									if(mappedInst != chnState.lastInst)
 										break;
 									m.note = note;
-									m.command = CMD_TONEPORTAMENTO;
-									m.param = 0xFF;
+									m.SetEffectCommand(CMD_TONEPORTA_DURATION, 0);
 									chnState.curPitchSlide = 0;
 									chnState.tonePortaRemain = 0;
+									chnState.retrigVibrato = chnState.retrigTremolo = true;
 									break;
 
 								// fine portamentos with range up to half a semitone
 								case SymEvent::PitchUp:
-									m.command = CMD_PORTAMENTOUP;
-									m.param = 0xF2;
+									m.SetEffectCommand(CMD_PORTAMENTOUP, 0xF2);
 									break;
 								case SymEvent::PitchDown:
-									m.command = CMD_PORTAMENTODOWN;
-									m.param = 0xF2;
+									m.SetEffectCommand(CMD_PORTAMENTODOWN, 0xF2);
 									break;
 								case SymEvent::PitchUp2:
-									m.command = CMD_PORTAMENTOUP;
-									m.param = 0xF4;
+									m.SetEffectCommand(CMD_PORTAMENTOUP, 0xF4);
 									break;
 								case SymEvent::PitchDown2:
-									m.command = CMD_PORTAMENTODOWN;
-									m.param = 0xF4;
+									m.SetEffectCommand(CMD_PORTAMENTODOWN, 0xF4);
 									break;
 								case SymEvent::PitchUp3:
-									m.command = CMD_PORTAMENTOUP;
-									m.param = 0xF8;
+									m.SetEffectCommand(CMD_PORTAMENTOUP, 0xF8);
 									break;
 								case SymEvent::PitchDown3:
-									m.command = CMD_PORTAMENTODOWN;
-									m.param = 0xF8;
+									m.SetEffectCommand(CMD_PORTAMENTODOWN, 0xF8);
 									break;
 								}
 							} else
@@ -1523,6 +1539,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 										m.instr = chnState.lastInst = mappedInst;
 										chnState.curPitchSlide = 0;
 										chnState.tonePortaRemain = 0;
+										chnState.retrigVibrato = chnState.retrigTremolo = true;
 									}
 
 									if(event.param > 0)
@@ -1539,12 +1556,11 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 							   applyVolume || chnState.calculatedVol != newVol)
 							{
 								chnState.calculatedVol = newVol;
-								m.command = CMD_CHANNELVOLUME;
-								m.param = newVol;
+								m.SetEffectCommand(CMD_CHANNELVOLUME, newVol);
 							}
 
 							// Key-On commands with stereo instruments are played on both channels - unless there's already some sort of event
-							if(event.note > 0 && (chn < m_nChannels - 1) && !(chn % 2u)
+							if(event.note > 0 && (chn < GetNumChannels() - 1) && !(chn % 2u)
 							   && origInst < instruments.size() && instruments[origInst].channel == SymInstrument::StereoL)
 							{
 								ModCommand &next = rowBase[chn + 1];
@@ -1576,17 +1592,21 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 							break;
 
 						case SymEvent::AddVolume:
-							m.command = m.param = 0;
+							m.command = CMD_NONE;
 							break;
 						case SymEvent::Tremolo:
-						{
-							// both tremolo speed and depth can go much higher than OpenMPT supports,
-							// but modules will probably use pretty sane, supportable values anyway
-							// TODO: handle very small nonzero params
-							uint8 speed = std::min<uint8>(15, event.inst >> 3);
-							uint8 depth = std::min<uint8>(15, event.param >> 3);
-							chnState.curTremolo = (speed << 4) | depth;
-						}
+							{
+								// both tremolo speed and depth can go much higher than OpenMPT supports,
+								// but modules will probably use pretty sane, supportable values anyway
+								// TODO: handle very small nonzero params
+								uint8 speed = std::min<uint8>(15, event.inst >> 3);
+								uint8 depth = std::min<uint8>(15, event.param >> 3);
+								chnState.curTremolo = (speed << 4) | depth;
+								if(chnState.curTremolo)
+									chnState.retrigTremolo = true;
+								else
+									m.SetEffectCommand(CMD_TREMOLO, 0);
+							}
 							break;
 
 							// pitch effects
@@ -1609,29 +1629,32 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 								const int distance = std::abs((note - chnState.lastNote) * 32);
 								chnState.curPitchSlide = 0;
 								m.note = chnState.lastNote = note;
-								m.command = CMD_TONEPORTAMENTO;
-								chnState.tonePortaAmt = m.param = mpt::saturate_cast<ModCommand::PARAM>(distance / (2 * event.param));
+								chnState.tonePortaAmt = mpt::saturate_cast<ModCommand::PARAM>(distance / (2 * event.param));
+								m.SetEffectCommand(CMD_TONEPORTAMENTO, chnState.tonePortaAmt);
 								chnState.tonePortaRemain = static_cast<uint16>(distance - std::min(distance, chnState.tonePortaAmt * (patternSpeed - 1)));
 							}
 							break;
 						case SymEvent::AddPitch:
 							// "The range (-128...127) is about 4 half notes."
-							m.command = m.param = 0;
+							m.command = CMD_NONE;
 							break;
 						case SymEvent::Vibrato:
-						{
-							// both vibrato speed and depth can go much higher than OpenMPT supports,
-							// but modules will probably use pretty sane, supportable values anyway
-							// TODO: handle very small nonzero params
-							uint8 speed = std::min<uint8>(15, event.inst >> 3);
-							uint8 depth = std::min<uint8>(15, event.param);
-							chnState.curVibrato = (speed << 4) | depth;
-						}
+							{
+								// both vibrato speed and depth can go much higher than OpenMPT supports,
+								// but modules will probably use pretty sane, supportable values anyway
+								// TODO: handle very small nonzero params
+								uint8 speed = std::min<uint8>(15, event.inst >> 3);
+								uint8 depth = std::min<uint8>(15, event.param);
+								chnState.curVibrato = (speed << 4) | depth;
+								if(chnState.curVibrato)
+									chnState.retrigVibrato = true;
+								else
+									m.SetEffectCommand(CMD_VIBRATO, 0);
+							}
 							break;
 						case SymEvent::AddHalfTone:
 							m.note = chnState.lastNote = Clamp(static_cast<uint8>(chnState.lastNote + event.param), NOTE_MIN, NOTE_MAX);
-							m.command = CMD_TONEPORTAMENTO;
-							m.param = 0xFF;
+							m.SetEffectCommand(CMD_TONEPORTA_DURATION, 0);
 							chnState.tonePortaRemain = 0;
 							break;
 
@@ -1641,21 +1664,22 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 						case SymEvent::DSPEcho:
 						case SymEvent::DSPDelay:
 #endif
-							if(macroMap.count(event))
+							if(auto it = macroMap.find(event); it != macroMap.end() && it->second != 0)
 							{
-								m.command = CMD_MIDI;
-								m.param = macroMap[event];
+								m.SetEffectCommand(CMD_MIDI, it->second);
 							} else if(macroMap.size() < m_MidiCfg.Zxx.size())
 							{
 								uint8 param = static_cast<uint8>(macroMap.size());
 								if(ConvertDSP(event, m_MidiCfg.Zxx[param], *this))
 								{
-									m.command = CMD_MIDI;
-									m.param = macroMap[event] = 0x80 | param;
+									m.SetEffectCommand(CMD_MIDI, macroMap[event] = 0x80 | param);
 
 									if(event.command == SymEvent::DSPEcho || event.command == SymEvent::DSPDelay)
 										useDSP = true;
 								}
+							} else if(uint8 param = MapToClosestMidiMacro(event, macroMap))
+							{
+								m.SetEffectCommand(CMD_MIDI, param);
 							}
 							break;
 
@@ -1665,14 +1689,13 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 							// The effect continues on the following rows until the correct amount is reached.
 							if(event.param < 1)
 								break;
-							m.command = CMD_RETRIG;
-							m.param = static_cast<uint8>(std::min(15, event.inst + 1));
-							chnState.retriggerRemain = event.param * (event.inst + 1u);
+							m.SetEffectCommand(CMD_RETRIG, static_cast<ModCommand::PARAM>(std::min(15, event.inst + 1)));
+							chnState.retriggerRemain = static_cast<uint16>(event.param * (event.inst + 1u));
 							break;
 
 						case SymEvent::SetSpeed:
-							m.command = CMD_SPEED;
-							m.param = patternSpeed = event.param ? event.param : 4u;
+							patternSpeed = event.param ? event.param : 4u;
+							m.SetEffectCommand(CMD_SPEED, patternSpeed);
 							break;
 
 							// TODO this applies a fade on the sample level
@@ -1690,17 +1713,14 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 								if(volL != chnState.channelVol)
 								{
 									chnState.channelVol = volL;
-
-									m.command = CMD_CHANNELVOLUME;
-									m.param   = chnState.calculatedVol = static_cast<uint8>(Util::muldivr_unsigned(chnState.lastVol, chnState.channelVol, 100));
+									chnState.calculatedVol = static_cast<uint8>(Util::muldivr_unsigned(chnState.lastVol, chnState.channelVol, 100));
+									m.SetEffectCommand(CMD_CHANNELVOLUME, chnState.calculatedVol);
 								}
-								if(event.note == 4 && chn < (m_nChannels - 1) && chnStates[chn + 1].channelVol != volR)
+								if(event.note == 4 && chn < (GetNumChannels() - 1) && chnStates[chn + 1].channelVol != volR)
 								{
 									chnStates[chn + 1].channelVol = volR;
-
-									ModCommand &next = rowBase[chn + 1];
-									next.command = CMD_CHANNELVOLUME;
-									next.param   = chnState.calculatedVol = static_cast<uint8>(Util::muldivr_unsigned(chnState.lastVol, chnState.channelVol, 100));
+									chnState.calculatedVol = static_cast<uint8>(Util::muldivr_unsigned(chnState.lastVol, chnState.channelVol, 100));
+									rowBase[chn + 1].SetEffectCommand(CMD_CHANNELVOLUME, chnState.calculatedVol);
 								}
 							}
 							break;
@@ -1737,8 +1757,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 								m.instr = chnState.lastInst = mappedInst;
 							if(event.command == SymEvent::ReplayFrom)
 							{
-								m.volcmd = VOLCMD_TONEPORTAMENTO;
-								m.vol = 1;
+								m.SetVolumeCommand(VOLCMD_TONEPORTAMENTO, 1);
 							}
 							// don't always add the command, because often FromAndPitch is used with offset 0
 							// to act as a key-on which doesn't cancel volume slides, etc
@@ -1747,8 +1766,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 								double sampleVib = 0.0;
 								if(chnState.sampleVibDepth)
 									sampleVib = chnState.sampleVibDepth * (std::sin(chnState.sampleVibPhase * (mpt::numbers::pi * 2.0 / 1024.0) + 1.5 * mpt::numbers::pi) - 1.0) / 4.0;
-								m.command = CMD_OFFSETPERCENTAGE;
-								m.param   = mpt::saturate_round<ModCommand::PARAM>(event.param + chnState.fromAdd + sampleVib);
+								m.SetEffectCommand(CMD_OFFSETPERCENTAGE, mpt::saturate_round<ModCommand::PARAM>(event.param + chnState.fromAdd + sampleVib));
 							}
 							chnState.tonePortaRemain = 0;
 							break;
@@ -1764,10 +1782,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 						{
 							chnState.retriggerRemain = std::max(chnState.retriggerRemain, static_cast<uint16>(patternSpeed)) - patternSpeed;
 							if(m.command == CMD_NONE)
-							{
-								m.command = CMD_RETRIG;
-								m.param = 0;
-							}
+								m.SetEffectCommand(CMD_RETRIG, 0);
 						}
 
 						// Handle fractional volume slides
@@ -1779,31 +1794,27 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 								if(patternSpeed > 1 && chnState.curVolSlideAmt >= (patternSpeed - 1))
 								{
 									uint8 slideAmt = std::min<uint8>(15, mpt::saturate_round<uint8>(chnState.curVolSlideAmt / (patternSpeed - 1)));
-									chnState.curVolSlideAmt -= slideAmt * (patternSpeed - 1);
+									chnState.curVolSlideAmt -= static_cast<float>(slideAmt * (patternSpeed - 1));
 									// normal slide up
-									m.command = CMD_CHANNELVOLSLIDE;
-									m.param = slideAmt << 4;
+									m.SetEffectCommand(CMD_CHANNELVOLSLIDE, slideAmt << 4);
 								} else if(chnState.curVolSlideAmt >= 1.0f)
 								{
 									uint8 slideAmt = std::min<uint8>(15, mpt::saturate_round<uint8>(chnState.curVolSlideAmt));
-									chnState.curVolSlideAmt -= slideAmt;
+									chnState.curVolSlideAmt -= static_cast<float>(slideAmt);
 									// fine slide up
-									m.command = CMD_CHANNELVOLSLIDE;
-									m.param = (slideAmt << 4) | 0x0F;
+									m.SetEffectCommand(CMD_CHANNELVOLSLIDE, (slideAmt << 4) | 0x0F);
 								} else if(patternSpeed > 1 && chnState.curVolSlideAmt <= -(patternSpeed - 1))
 								{
 									uint8 slideAmt = std::min<uint8>(15, mpt::saturate_round<uint8>(-chnState.curVolSlideAmt / (patternSpeed - 1)));
-									chnState.curVolSlideAmt += slideAmt * (patternSpeed - 1);
+									chnState.curVolSlideAmt += static_cast<float>(slideAmt * (patternSpeed - 1));
 									// normal slide down
-									m.command = CMD_CHANNELVOLSLIDE;
-									m.param = slideAmt;
+									m.SetEffectCommand(CMD_CHANNELVOLSLIDE, slideAmt);
 								} else if(chnState.curVolSlideAmt <= -1.0f)
 								{
 									uint8 slideAmt = std::min<uint8>(14, mpt::saturate_round<uint8>(-chnState.curVolSlideAmt));
-									chnState.curVolSlideAmt += slideAmt;
+									chnState.curVolSlideAmt += static_cast<float>(slideAmt);
 									// fine slide down
-									m.command = CMD_CHANNELVOLSLIDE;
-									m.param = slideAmt | 0xF0;
+									m.SetEffectCommand(CMD_CHANNELVOLSLIDE, slideAmt | 0xF0);
 								}
 							}
 						}
@@ -1816,31 +1827,27 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 								if(patternSpeed > 1 && chnState.curPitchSlideAmt >= (patternSpeed - 1))
 								{
 									uint8 slideAmt = std::min<uint8>(0xDF, mpt::saturate_round<uint8>(chnState.curPitchSlideAmt / (patternSpeed - 1)));
-									chnState.curPitchSlideAmt -= slideAmt * (patternSpeed - 1);
+									chnState.curPitchSlideAmt -= static_cast<float>(slideAmt * (patternSpeed - 1));
 									// normal slide up
-									m.command = CMD_PORTAMENTOUP;
-									m.param = slideAmt;
+									m.SetEffectCommand(CMD_PORTAMENTOUP, slideAmt);
 								} else if(chnState.curPitchSlideAmt >= 1.0f)
 								{
 									uint8 slideAmt = std::min<uint8>(15, mpt::saturate_round<uint8>(chnState.curPitchSlideAmt));
-									chnState.curPitchSlideAmt -= slideAmt;
+									chnState.curPitchSlideAmt -= static_cast<float>(slideAmt);
 									// fine slide up
-									m.command = CMD_PORTAMENTOUP;
-									m.param = slideAmt | 0xF0;
+									m.SetEffectCommand(CMD_PORTAMENTOUP, slideAmt | 0xF0);
 								} else if(patternSpeed > 1 && chnState.curPitchSlideAmt <= -(patternSpeed - 1))
 								{
 									uint8 slideAmt = std::min<uint8>(0xDF, mpt::saturate_round<uint8>(-chnState.curPitchSlideAmt / (patternSpeed - 1)));
-									chnState.curPitchSlideAmt += slideAmt * (patternSpeed - 1);
+									chnState.curPitchSlideAmt += static_cast<float>(slideAmt * (patternSpeed - 1));
 									// normal slide down
-									m.command = CMD_PORTAMENTODOWN;
-									m.param = slideAmt;
+									m.SetEffectCommand(CMD_PORTAMENTODOWN, slideAmt);
 								} else if(chnState.curPitchSlideAmt <= -1.0f)
 								{
 									uint8 slideAmt = std::min<uint8>(14, mpt::saturate_round<uint8>(-chnState.curPitchSlideAmt));
-									chnState.curPitchSlideAmt += slideAmt;
+									chnState.curPitchSlideAmt += static_cast<float>(slideAmt);
 									// fine slide down
-									m.command = CMD_PORTAMENTODOWN;
-									m.param = slideAmt | 0xF0;
+									m.SetEffectCommand(CMD_PORTAMENTODOWN, slideAmt | 0xF0);
 								}
 							}
 							// TODO: use volume column if effect column is occupied
@@ -1849,28 +1856,26 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 								if(patternSpeed > 1 && chnState.curPitchSlideAmt / 4 >= (patternSpeed - 1))
 								{
 									uint8 slideAmt = std::min<uint8>(9, mpt::saturate_round<uint8>(chnState.curPitchSlideAmt / (patternSpeed - 1)) / 4);
-									chnState.curPitchSlideAmt -= slideAmt * (patternSpeed - 1) * 4;
-									m.volcmd = VOLCMD_PORTAUP;
-									m.vol = slideAmt;
+									chnState.curPitchSlideAmt -= static_cast<float>(slideAmt * (patternSpeed - 1) * 4);
+									m.SetVolumeCommand(VOLCMD_PORTAUP, slideAmt);
 								} else if(patternSpeed > 1 && chnState.curPitchSlideAmt / 4 <= -(patternSpeed - 1))
 								{
 									uint8 slideAmt = std::min<uint8>(9, mpt::saturate_round<uint8>(-chnState.curPitchSlideAmt / (patternSpeed - 1)) / 4);
-									chnState.curPitchSlideAmt += slideAmt * (patternSpeed - 1) * 4;
-									m.volcmd = VOLCMD_PORTADOWN;
-									m.vol = slideAmt;
+									chnState.curPitchSlideAmt += static_cast<float>(slideAmt * (patternSpeed - 1) * 4);
+									m.SetVolumeCommand(VOLCMD_PORTADOWN, slideAmt);
 								}
 							}
 						}
 						// Vibrato and Tremolo
-						if(m.command == CMD_NONE && chnState.curVibrato != 0)
+						if(m.command == CMD_NONE && chnState.curVibrato && chnState.retrigVibrato)
 						{
-							m.command = CMD_VIBRATO;
-							m.param = chnState.curVibrato;
+							m.SetEffectCommand(CMD_VIBRATO, chnState.curVibrato);
+							chnState.retrigVibrato = false;
 						}
-						if(m.command == CMD_NONE && chnState.curTremolo != 0)
+						if(m.command == CMD_NONE && chnState.curTremolo && chnState.retrigTremolo)
 						{
-							m.command = CMD_TREMOLO;
-							m.param = chnState.curTremolo;
+							m.SetEffectCommand(CMD_TREMOLO, chnState.curTremolo);
+							chnState.retrigTremolo = false;
 						}
 						// Tone Portamento
 						if(m.command != CMD_TONEPORTAMENTO && chnState.tonePortaRemain)
@@ -1907,7 +1912,7 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 	if(useDSP)
 	{
 		SNDMIXPLUGIN &plugin = m_MixPlugins[0];
-		plugin.Destroy();
+		mpt::reconstruct(plugin);
 		memcpy(&plugin.Info.dwPluginId1, "SymM", 4);
 		memcpy(&plugin.Info.dwPluginId2, "Echo", 4);
 		plugin.Info.routingFlags    = SNDMIXPLUGININFO::irAutoSuspend;
@@ -1924,21 +1929,20 @@ bool CSoundFile::ReadSymMOD(FileReader &file, ModLoadingFlags loadFlags)
 #endif // NO_PLUGINS
 
 	// Channel panning
-	for(CHANNELINDEX chn = 0; chn < m_nChannels; chn++)
+	for(CHANNELINDEX chn = 0; chn < GetNumChannels(); chn++)
 	{
-		InitChannel(chn);
 		ChnSettings[chn].nPan = (chn & 1) ? 256 : 0;
 		ChnSettings[chn].nMixPlugin = useDSP ? 1 : 0;  // For MIDI macros controlling the echo DSP
 	}
 
-	m_modFormat.formatName = U_("Symphonie");
-	m_modFormat.type = U_("symmod");
+	m_modFormat.formatName = UL_("Symphonie");
+	m_modFormat.type = UL_("symmod");
 	if(!isSymphoniePro)
-		m_modFormat.madeWithTracker = U_("Symphonie");  // or Symphonie Jr
+		m_modFormat.madeWithTracker = UL_("Symphonie");  // or Symphonie Jr
 	else if(instruments.size() <= 128)
-		m_modFormat.madeWithTracker = U_("Symphonie Pro");
+		m_modFormat.madeWithTracker = UL_("Symphonie Pro");
 	else
-		m_modFormat.madeWithTracker = U_("Symphonie Pro 256");
+		m_modFormat.madeWithTracker = UL_("Symphonie Pro 256");
 	m_modFormat.charset = mpt::Charset::Amiga_no_C1;
 
 	return true;
