@@ -9,8 +9,8 @@
 
 
 #include "stdafx.h"
-#include "Sndfile.h"
 #include "ModInstrument.h"
+#include "Sndfile.h"
 
 
 OPENMPT_NAMESPACE_BEGIN
@@ -41,6 +41,12 @@ void InstrumentEnvelope::Convert(MODTYPE fromType, MODTYPE toType)
 			// So we have to disable the sustain loop if it was behind the normal loop.
 			dwFlags.reset(ENV_SUSTAIN);
 		}
+		if(!dwFlags[ENV_LOOP | ENV_SUSTAIN])
+		{
+			// XM has no automatic fade-out behaviour at the end of the envelope.
+			dwFlags.set(ENV_SUSTAIN);
+			nSustainStart = nSustainEnd = LastPoint();
+		}
 
 		// XM -> IT / MPTM: Shorten loop by one tick by inserting bogus point
 		if(nLoopEnd > nLoopStart && dwFlags[ENV_LOOP] && nLoopEnd < size())
@@ -48,7 +54,7 @@ void InstrumentEnvelope::Convert(MODTYPE fromType, MODTYPE toType)
 			if(at(nLoopEnd).tick - 1 > at(nLoopEnd - 1).tick)
 			{
 				// Insert an interpolated point just before the loop point.
-				EnvelopeNode::tick_t tick = at(nLoopEnd).tick - 1u;
+				EnvelopeNode::tick_t tick = static_cast<EnvelopeNode::tick_t>(at(nLoopEnd).tick - 1u);
 				auto interpolatedValue = static_cast<EnvelopeNode::value_t>(GetValueFromPosition(tick, 64));
 				insert(begin() + nLoopEnd, EnvelopeNode(tick, interpolatedValue));
 			} else
@@ -73,11 +79,11 @@ int32 InstrumentEnvelope::GetValueFromPosition(int position, int32 rangeOut, int
 	if(empty())
 		return 0;
 
-	uint32 pt = size() - 1u;
+	uint32 pt = LastPoint();
 	const int32 ENV_PRECISION = 1 << 16;
 
 	// Checking where current 'tick' is relative to the envelope points.
-	for(uint32 i = 0; i < size() - 1u; i++)
+	for(uint32 i = 0; i < LastPoint(); i++)
 	{
 		if (position <= at(i).tick)
 		{
@@ -129,12 +135,12 @@ void InstrumentEnvelope::Sanitize(uint8 maxValue)
 			it->tick = std::max(it->tick, (it - 1)->tick);
 			LimitMax(it->value, maxValue);
 		}
-		LimitMax(nLoopEnd, static_cast<decltype(nLoopEnd)>(size() - 1));
+		LimitMax(nLoopEnd, LastPoint());
 		LimitMax(nLoopStart, nLoopEnd);
-		LimitMax(nSustainEnd, static_cast<decltype(nSustainEnd)>(size() - 1));
+		LimitMax(nSustainEnd, LastPoint());
 		LimitMax(nSustainStart, nSustainEnd);
 		if(nReleaseNode != ENV_RELEASE_NODE_UNSET)
-			LimitMax(nReleaseNode, static_cast<decltype(nReleaseNode)>(size() - 1));
+			LimitMax(nReleaseNode, LastPoint());
 	} else
 	{
 		nLoopStart = 0;
@@ -146,24 +152,11 @@ void InstrumentEnvelope::Sanitize(uint8 maxValue)
 }
 
 
-ModInstrument::ModInstrument(SAMPLEINDEX sample)
-{
-	SetCutoff(0, false);
-	SetResonance(0, false);
-
-	pitchToTempoLock.Set(0);
-
-	pTuning = CSoundFile::GetDefaultTuning();
-
-	AssignSample(sample);
-	ResetNoteMap();
-}
-
-
 // Translate instrument properties between two given formats.
 void ModInstrument::Convert(MODTYPE fromType, MODTYPE toType)
 {
 	MPT_UNREFERENCED_PARAMETER(fromType);
+	synth.Clear();
 
 	if(toType & MOD_TYPE_XM)
 	{
@@ -220,10 +213,11 @@ void ModInstrument::Convert(MODTYPE fromType, MODTYPE toType)
 		}
 	}
 
-	// Limit fadeout length for IT
+	// Limit fadeout length and precision for IT
 	if(toType & MOD_TYPE_IT)
 	{
 		LimitMax(nFadeOut, 8192u);
+		nFadeOut = ((nFadeOut + 16) / 32) * 32;
 	}
 
 	// MPT-specific features - remove instrument tunings, Pitch/Tempo Lock, cutoff / resonance swing and filter mode for other formats
@@ -295,11 +289,12 @@ void ModInstrument::Sanitize(MODTYPE modType)
 	MPT_UNREFERENCED_PARAMETER(modType);
 	const uint8 range = ENVELOPE_MAX;
 #else
-	const uint8 range = modType == MOD_TYPE_AMS ? uint8_max : ENVELOPE_MAX;
+	const uint8 range = modType == MOD_TYPE_AMS ? uint8_max : uint8(ENVELOPE_MAX);
 #endif
 	VolEnv.Sanitize();
 	PanEnv.Sanitize();
 	PitchEnv.Sanitize(range);
+	synth.Sanitize();
 
 	for(size_t i = 0; i < std::size(NoteMap); i++)
 	{
