@@ -6,14 +6,15 @@
  *          It seems like no other game used the AM(FF) format.
  *          RIFF AM is the newer version of the format, generally following the RIFF "standard" closely.
  * Authors: Johannes Schultz (OpenMPT port, reverse engineering + loader implementation of the instrument format)
- *          Chris Moeller (foo_dumb - this is almost a complete port of his code, thanks)
+ *          Largely based on the J2B loader of kode54's DUMB fork
  * The OpenMPT source code is released under the BSD license. Read LICENSE for more details.
  */
 
 
 #include "stdafx.h"
 #include "Loaders.h"
-#include "ChunkReader.h"
+
+#include "mpt/io/base.hpp"
 
 #if defined(MPT_WITH_ZLIB)
 #include <zlib.h>
@@ -22,11 +23,16 @@
 #endif
 
 
+#ifdef MPT_ALL_LOGGING
+#define J2B_LOG
+#endif
+
+
 OPENMPT_NAMESPACE_BEGIN
 
 
 // First off, a nice vibrato translation LUT.
-static const uint8 j2bAutoVibratoTrans[] = 
+static constexpr VibratoType j2bAutoVibratoTrans[] = 
 {
 	VIB_SINE, VIB_SQUARE, VIB_RAMP_UP, VIB_RAMP_DOWN, VIB_RANDOM,
 };
@@ -37,15 +43,18 @@ struct J2BFileHeader
 {
 	// Magic Bytes
 	// 32-Bit J2B header identifiers
-	static const uint32 magicDEADBEAF = 0xAFBEADDEu;
-	static const uint32 magicDEADBABE = 0xBEBAADDEu;
+	enum : uint32
+	{
+		magicDEADBEAF = 0xAFBEADDEu,
+		magicDEADBABE = 0xBEBAADDEu
+	};
 
-	char     signature[4];		// MUSE
-	uint32le deadbeaf;			// 0xDEADBEAF (AM) or 0xDEADBABE (AMFF)
-	uint32le fileLength;		// complete filesize
-	uint32le crc32;				// checksum of the compressed data block
-	uint32le packedLength;		// length of the compressed data block
-	uint32le unpackedLength;	// length of the decompressed module
+	char     signature[4];    // MUSE
+	uint32le deadbeaf;        // 0xDEADBEAF (AM) or 0xDEADBABE (AMFF)
+	uint32le fileLength;      // complete filesize
+	uint32le crc32;           // checksum of the compressed data block
+	uint32le packedLength;    // length of the compressed data block
+	uint32le unpackedLength;  // length of the decompressed module
 };
 
 MPT_BINARY_STRUCT(J2BFileHeader, 24)
@@ -58,21 +67,21 @@ struct AMFFRiffChunk
 	// 32-Bit chunk identifiers
 	enum ChunkIdentifiers
 	{
-		idRIFF	= MAGIC4LE('R','I','F','F'),
-		idAMFF	= MAGIC4LE('A','M','F','F'),
-		idAM__	= MAGIC4LE('A','M',' ',' '),
-		idMAIN	= MAGIC4LE('M','A','I','N'),
-		idINIT	= MAGIC4LE('I','N','I','T'),
-		idORDR	= MAGIC4LE('O','R','D','R'),
-		idPATT	= MAGIC4LE('P','A','T','T'),
-		idINST	= MAGIC4LE('I','N','S','T'),
-		idSAMP	= MAGIC4LE('S','A','M','P'),
-		idAI__	= MAGIC4LE('A','I',' ',' '),
-		idAS__	= MAGIC4LE('A','S',' ',' '),
+		idRIFF = MagicLE("RIFF"),
+		idAMFF = MagicLE("AMFF"),
+		idAM__ = MagicLE("AM  "),
+		idMAIN = MagicLE("MAIN"),
+		idINIT = MagicLE("INIT"),
+		idORDR = MagicLE("ORDR"),
+		idPATT = MagicLE("PATT"),
+		idINST = MagicLE("INST"),
+		idSAMP = MagicLE("SAMP"),
+		idAI__ = MagicLE("AI  "),
+		idAS__ = MagicLE("AS  "),
 	};
 
-	uint32le id;		// See ChunkIdentifiers
-	uint32le length;	// Chunk size without header
+	uint32le id;      // See ChunkIdentifiers
+	uint32le length;  // Chunk size without header
 
 	size_t GetLength() const
 	{
@@ -102,8 +111,8 @@ struct AMFFMainChunk
 	uint8le  channels;
 	uint8le  speed;
 	uint8le  tempo;
-	uint16le minPeriod;	// 16x Amiga periods, but we should ignore them - otherwise some high notes in Medivo.j2b won't sound correct.
-	uint16le maxPeriod;	// Ditto
+	uint16le minPeriod;  // 16x Amiga periods, but we should ignore them - otherwise some high notes in Medivo.j2b won't sound correct.
+	uint16le maxPeriod;  // Ditto
 	uint8le  globalvolume;
 };
 
@@ -124,14 +133,14 @@ struct AMFFEnvelope
 	struct EnvPoint
 	{
 		uint16le tick;
-		uint8le  value;	// 0...64
+		uint8le  value;  // 0...64
 	};
 
-	uint8le envFlags;			// high nibble = pan env flags, low nibble = vol env flags (both nibbles work the same way)
-	uint8le envNumPoints;		// high nibble = pan env length, low nibble = vol env length
-	uint8le envSustainPoints;	// you guessed it... high nibble = pan env sustain point, low nibble = vol env sustain point
-	uint8le envLoopStarts;		// I guess you know the pattern now.
-	uint8le envLoopEnds;		// same here.
+	uint8le envFlags;          // high nibble = pan env flags, low nibble = vol env flags (both nibbles work the same way)
+	uint8le envNumPoints;      // high nibble = pan env length, low nibble = vol env length
+	uint8le envSustainPoints;  // you guessed it... high nibble = pan env sustain point, low nibble = vol env sustain point
+	uint8le envLoopStarts;     // I guess you know the pattern now.
+	uint8le envLoopEnds;       // same here.
 	EnvPoint volEnv[10];
 	EnvPoint panEnv[10];
 
@@ -183,8 +192,8 @@ MPT_BINARY_STRUCT(AMFFEnvelope, 65)
 // AMFF instrument header (old format)
 struct AMFFInstrumentHeader
 {
-	uint8le  unknown;		// 0x00
-	uint8le  index;			// actual instrument number
+	uint8le  unknown;  // 0x00
+	uint8le  index;    // actual instrument number
 	char     name[28];
 	uint8le  numSamples;
 	uint8le  sampleMap[120];
@@ -198,12 +207,12 @@ struct AMFFInstrumentHeader
 	// Convert instrument data to OpenMPT's internal format.
 	void ConvertToMPT(ModInstrument &mptIns, SAMPLEINDEX baseSample)
 	{
-		mpt::String::Read<mpt::String::maybeNullTerminated>(mptIns.name, name);
+		mptIns.name = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, name);
 
-		STATIC_ASSERT(CountOf(sampleMap) <= CountOf(mptIns.Keyboard));
-		for(size_t i = 0; i < CountOf(sampleMap); i++)
+		static_assert(mpt::array_size<decltype(sampleMap)>::size <= mpt::array_size<decltype(mptIns.Keyboard)>::size);
+		for(size_t i = 0; i < std::size(sampleMap); i++)
 		{
-			mptIns.Keyboard[i] = sampleMap[i] + baseSample + 1;
+			mptIns.Keyboard[i] = static_cast<SAMPLEINDEX>(sampleMap[i] + baseSample + 1);
 		}
 
 		mptIns.nFadeOut = fadeout << 5;
@@ -221,16 +230,16 @@ struct AMFFSampleHeader
 	// Sample flags (also used for RIFF AM)
 	enum SampleFlags
 	{
-		smp16Bit	= 0x04,
-		smpLoop		= 0x08,
-		smpPingPong	= 0x10,
-		smpPanning	= 0x20,
-		smpExists	= 0x80,
+		smp16Bit    = 0x04,
+		smpLoop     = 0x08,
+		smpPingPong = 0x10,
+		smpPanning  = 0x20,
+		smpExists   = 0x80,
 		// some flags are still missing... what is e.g. 0x8000?
 	};
 
-	uint32le id;		// "SAMP"
-	uint32le chunkSize;	// header + sample size
+	uint32le id;         // "SAMP"
+	uint32le chunkSize;  // header + sample size
 	char     name[28];
 	uint8le  pan;
 	uint8le  volume;
@@ -254,7 +263,7 @@ struct AMFFSampleHeader
 		mptSmp.nLoopEnd = loopEnd;
 		mptSmp.nC5Speed = sampleRate;
 
-		if(instrHeader.vibratoType < CountOf(j2bAutoVibratoTrans))
+		if(instrHeader.vibratoType < std::size(j2bAutoVibratoTrans))
 			mptSmp.nVibType = j2bAutoVibratoTrans[instrHeader.vibratoType];
 		mptSmp.nVibSweep = static_cast<uint8>(instrHeader.vibratoSweep);
 		mptSmp.nVibRate = static_cast<uint8>(instrHeader.vibratoRate / 16);
@@ -295,16 +304,16 @@ struct AMEnvelope
 	struct EnvPoint
 	{
 		uint16le tick;
-		uint16le value;
+		int16le value;
 	};
 
 	uint16le flags;
-	uint8le  numPoints;		// actually, it's num. points - 1, and 0xFF if there is no envelope
+	uint8le  numPoints;  // actually, it's num. points - 1, and 0xFF if there is no envelope
 	uint8le  sustainPoint;
 	uint8le  loopStart;
 	uint8le  loopEnd;
 	EnvPoint values[10];
-	uint16le fadeout;		// why is this here? it's only needed for the volume envelope...
+	uint16le fadeout;  // why is this here? it's only needed for the volume envelope...
 
 	// Convert envelope data to OpenMPT's internal format.
 	void ConvertToMPT(InstrumentEnvelope &mptEnv, EnvelopeType envType) const
@@ -319,6 +328,23 @@ struct AMEnvelope
 		mptEnv.nLoopStart = loopStart;
 		mptEnv.nLoopEnd = loopEnd;
 
+		int32 scale = 0, offset = 0;
+		switch(envType)
+		{
+		case ENV_VOLUME:  // 0....32767
+		default:
+			scale = 32767 / ENVELOPE_MAX;
+			break;
+		case ENV_PITCH:  // -4096....4096
+			scale = 8192 / ENVELOPE_MAX;
+			offset = 4096;
+			break;
+		case ENV_PANNING:  // -32768...32767
+			scale = 65536 / ENVELOPE_MAX;
+			offset = 32768;
+			break;
+		}
+
 		for(uint32 i = 0; i < mptEnv.size(); i++)
 		{
 			mptEnv[i].tick = values[i].tick >> 4;
@@ -327,21 +353,9 @@ struct AMEnvelope
 			else if(mptEnv[i].tick < mptEnv[i - 1].tick)
 				mptEnv[i].tick = mptEnv[i - 1].tick + 1;
 
-			const uint16 val = values[i].value;
-			switch(envType)
-			{
-			case ENV_VOLUME:	// 0....32767
-			default:
-				mptEnv[i].value = (uint8)((val + 1) >> 9);
-				break;
-			case ENV_PITCH:		// -4096....4096
-				mptEnv[i].value = (uint8)((((int16)val) + 0x1001) >> 7);
-				break;
-			case ENV_PANNING:	// -32768...32767
-				mptEnv[i].value = (uint8)((((int16)val) + 0x8001) >> 10);
-				break;
-			}
-			Limit(mptEnv[i].value, uint8(ENVELOPE_MIN), uint8(ENVELOPE_MAX));
+			int32 val = values[i].value + offset;
+			val = (val + scale / 2) / scale;
+			mptEnv[i].value = static_cast<EnvelopeNode::value_t>(std::clamp(val, int32(ENVELOPE_MIN), int32(ENVELOPE_MAX)));
 		}
 
 		mptEnv.dwFlags.set(ENV_ENABLED, (flags & AMFFEnvelope::envEnabled) != 0);
@@ -350,16 +364,15 @@ struct AMEnvelope
 	}
 };
 
-MPT_BINARY_STRUCT(AMEnvelope::EnvPoint, 4)
 MPT_BINARY_STRUCT(AMEnvelope, 48)
 
 
 // AM instrument header (new format)
 struct AMInstrumentHeader
 {
-	uint32le headSize;	// Header size (i.e. the size of this struct)
-	uint8le  unknown1;	// 0x00
-	uint8le  index;		// Actual instrument number
+	uint32le headSize;  // Header size (i.e. the size of this struct)
+	uint8le  unknown1;  // 0x00
+	uint8le  index;     // Actual instrument number
 	char     name[32];
 	uint8le  sampleMap[128];
 	uint8le  vibratoType;
@@ -375,12 +388,12 @@ struct AMInstrumentHeader
 	// Convert instrument data to OpenMPT's internal format.
 	void ConvertToMPT(ModInstrument &mptIns, SAMPLEINDEX baseSample)
 	{
-		mpt::String::Read<mpt::String::maybeNullTerminated>(mptIns.name, name);
+		mptIns.name = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, name);
 
-		STATIC_ASSERT(CountOf(sampleMap) <= CountOf(mptIns.Keyboard));
-		for(uint8 i = 0; i < CountOf(sampleMap); i++)
+		static_assert(mpt::array_size<decltype(sampleMap)>::size <= mpt::array_size<decltype(mptIns.Keyboard)>::size);
+		for(uint8 i = 0; i < std::size(sampleMap); i++)
 		{
-			mptIns.Keyboard[i] = sampleMap[i] + baseSample + 1;
+			mptIns.Keyboard[i] = static_cast<SAMPLEINDEX>(sampleMap[i] + baseSample + 1);
 		}
 
 		mptIns.nFadeOut = volEnv.fadeout << 5;
@@ -402,12 +415,12 @@ MPT_BINARY_STRUCT(AMInstrumentHeader, 326)
 // AM sample header (new format)
 struct AMSampleHeader
 {
-	uint32le headSize;		// Header size (i.e. the size of this struct), apparently not including headSize.
+	uint32le headSize;  // Header size (i.e. the size of this struct), apparently not including headSize.
 	char     name[32];
 	uint16le pan;
 	uint16le volume;
 	uint16le flags;
-	uint16le unknown;		// 0x0000 / 0x0080?
+	uint16le unknown;  // 0x0000 / 0x0080?
 	uint32le length;
 	uint32le loopStart;
 	uint32le loopEnd;
@@ -417,15 +430,15 @@ struct AMSampleHeader
 	void ConvertToMPT(AMInstrumentHeader &instrHeader, ModSample &mptSmp) const
 	{
 		mptSmp.Initialize();
-		mptSmp.nPan = std::min<uint16>(pan, 32767) * 256 / 32767;
-		mptSmp.nVolume = std::min<uint16>(volume, 32767) * 256 / 32767;
+		mptSmp.nPan = static_cast<uint16>(std::min(pan.get(), uint16(32767)) * 256 / 32767);
+		mptSmp.nVolume = static_cast<uint16>(std::min(volume.get(), uint16(32767)) * 256 / 32767);
 		mptSmp.nGlobalVol = 64;
 		mptSmp.nLength = length;
 		mptSmp.nLoopStart = loopStart;
 		mptSmp.nLoopEnd = loopEnd;
 		mptSmp.nC5Speed = sampleRate;
 
-		if(instrHeader.vibratoType < CountOf(j2bAutoVibratoTrans))
+		if(instrHeader.vibratoType < std::size(j2bAutoVibratoTrans))
 			mptSmp.nVibType = j2bAutoVibratoTrans[instrHeader.vibratoType];
 		mptSmp.nVibSweep = static_cast<uint8>(instrHeader.vibratoSweep);
 		mptSmp.nVibRate = static_cast<uint8>(instrHeader.vibratoRate / 16);
@@ -464,7 +477,7 @@ MPT_BINARY_STRUCT(AMSampleHeader, 60)
 static bool ConvertAMPattern(FileReader chunk, PATTERNINDEX pat, bool isAM, CSoundFile &sndFile)
 {
 	// Effect translation LUT
-	static const EffectCommand amEffTrans[] =
+	static constexpr EffectCommand amEffTrans[] =
 	{
 		CMD_ARPEGGIO, CMD_PORTAMENTOUP, CMD_PORTAMENTODOWN, CMD_TONEPORTAMENTO,
 		CMD_VIBRATO, CMD_TONEPORTAVOL, CMD_VIBRATOVOL, CMD_TREMOLO,
@@ -477,31 +490,24 @@ static bool ConvertAMPattern(FileReader chunk, PATTERNINDEX pat, bool isAM, CSou
 
 	enum
 	{
-		rowDone		= 0,		// Advance to next row
-		channelMask	= 0x1F,		// Mask for retrieving channel information
-		volFlag		= 0x20,		// Volume effect present
-		noteFlag	= 0x40,		// Note + instr present
-		effectFlag	= 0x80,		// Effect information present
-		dataFlag	= 0xE0,		// Channel data present
+		rowDone     = 0x00,  // Advance to next row
+		channelMask = 0x1F,  // Mask for retrieving channel information
+		volFlag     = 0x20,  // Volume effect present
+		noteFlag    = 0x40,  // Note + instr present
+		effectFlag  = 0x80,  // Effect information present
+		dataFlag    = 0xE0,  // Channel data present
 	};
 
 	if(chunk.NoBytesLeft())
-	{
 		return false;
-	}
 
-	ROWINDEX numRows = Clamp(static_cast<ROWINDEX>(chunk.ReadUint8()) + 1, ROWINDEX(1), MAX_PATTERN_ROWS);
+	const ROWINDEX numRows = std::min(static_cast<ROWINDEX>(chunk.ReadUint8() + 1), MAX_PATTERN_ROWS);
 
 	if(!sndFile.Patterns.Insert(pat, numRows))
 		return false;
 
-	const CHANNELINDEX channels = sndFile.GetNumChannels();
-	if(channels == 0)
-		return false;
-
-	PatternRow rowBase = sndFile.Patterns[pat].GetRow(0);
+	const CHANNELINDEX lastChannel = sndFile.GetNumChannels() - 1;
 	ROWINDEX row = 0;
-
 	while(row < numRows && chunk.CanRead(1))
 	{
 		const uint8 flags = chunk.ReadUint8();
@@ -509,98 +515,97 @@ static bool ConvertAMPattern(FileReader chunk, PATTERNINDEX pat, bool isAM, CSou
 		if(flags == rowDone)
 		{
 			row++;
-			rowBase = sndFile.Patterns[pat].GetRow(row);
 			continue;
 		}
+		if(!(flags & dataFlag))
+			continue;
 
-		ModCommand &m = rowBase[std::min<CHANNELINDEX>((flags & channelMask), channels - 1)];
+		ModCommand &m = *sndFile.Patterns[pat].GetpModCommand(row, std::min(static_cast<CHANNELINDEX>(flags & channelMask), lastChannel));
 
-		if(flags & dataFlag)
+		if(flags & effectFlag) // effect
 		{
-			if(flags & effectFlag) // effect
+			const auto [param, command] = chunk.ReadArray<uint8, 2>();
+			if(command < std::size(amEffTrans))
 			{
-				m.param = chunk.ReadUint8();
-				uint8 command = chunk.ReadUint8();
+				m.SetEffectCommand(amEffTrans[command], param);
+			} else
+			{
+#ifdef J2B_LOG
+				MPT_LOG_GLOBAL(LogDebug, "J2B", MPT_UFORMAT("J2B: Unknown command: 0x{}, param 0x{}")(mpt::ufmt::HEX0<2>(command), mpt::ufmt::HEX0<2>(m.param)));
+#endif
+				m.command = CMD_NONE;
+			}
 
-				if(command < CountOf(amEffTrans))
+			// Handling special commands
+			switch(m.command)
+			{
+			case CMD_ARPEGGIO:
+				if(m.param == 0) m.command = CMD_NONE;
+				break;
+			case CMD_VOLUME:
+				if(m.volcmd == VOLCMD_NONE && !(flags & volFlag))
 				{
-					// command translation
-					m.command = amEffTrans[command];
-				} else
-				{
-#ifdef DEBUG
-					Log(mpt::format("J2B: Unknown command: 0x%1, param 0x%2")(mpt::fmt::HEX0<2>(command), mpt::fmt::HEX0<2>(m.param)));
-#endif // DEBUG
+					m.SetVolumeCommand(VOLCMD_VOLUME, Clamp(m.param, uint8(0), uint8(64)));
 					m.command = CMD_NONE;
 				}
-
-				// Handling special commands
-				switch(m.command)
+				break;
+			case CMD_TONEPORTAVOL:
+			case CMD_VIBRATOVOL:
+			case CMD_VOLUMESLIDE:
+			case CMD_GLOBALVOLSLIDE:
+			case CMD_PANNINGSLIDE:
+				if(m.param & 0xF0)
+					m.param &= 0xF0;
+				break;
+			case CMD_PANNING8:
+				if(m.param <= 0x80)
+					m.param = mpt::saturate_cast<ModCommand::PARAM>(m.param * 2);
+				else if(m.param == 0xA4)
+					m.SetEffectCommand(CMD_S3MCMDEX, 0x91u);
+				break;
+			case CMD_PATTERNBREAK:
+				m.param = static_cast<ModCommand::PARAM>(((m.param >> 4) * 10u) + (m.param & 0x0Fu));
+				break;
+			case CMD_MODCMDEX:
+				m.ExtendedMODtoS3MEffect();
+				break;
+			case CMD_TEMPO:
+				if(m.param <= 0x1F)
+					m.command = CMD_SPEED;
+				break;
+			case CMD_XFINEPORTAUPDOWN:
+				switch(m.param & 0xF0)
 				{
-				case CMD_ARPEGGIO:
-					if(m.param == 0) m.command = CMD_NONE;
+				case 0x10:
+					m.command = CMD_PORTAMENTOUP;
 					break;
-				case CMD_VOLUME:
-					if(m.volcmd == VOLCMD_NONE)
-					{
-						m.volcmd = VOLCMD_VOLUME;
-						m.vol = Clamp(m.param, uint8(0), uint8(64));
-						m.command = CMD_NONE;
-						m.param = 0;
-					}
-					break;
-				case CMD_TONEPORTAVOL:
-				case CMD_VIBRATOVOL:
-				case CMD_VOLUMESLIDE:
-				case CMD_GLOBALVOLSLIDE:
-				case CMD_PANNINGSLIDE:
-					if (m.param & 0xF0) m.param &= 0xF0;
-					break;
-				case CMD_PANNING8:
-					if(m.param <= 0x80) m.param = MIN(m.param << 1, 0xFF);
-					else if(m.param == 0xA4) {m.command = CMD_S3MCMDEX; m.param = 0x91;}
-					break;
-				case CMD_PATTERNBREAK:
-					m.param = ((m.param >> 4) * 10) + (m.param & 0x0F);
-					break;
-				case CMD_MODCMDEX:
-					m.ExtendedMODtoS3MEffect();
-					break;
-				case CMD_TEMPO:
-					if(m.param <= 0x1F) m.command = CMD_SPEED;
-					break;
-				case CMD_XFINEPORTAUPDOWN:
-					switch(m.param & 0xF0)
-					{
-					case 0x10:
-						m.command = CMD_PORTAMENTOUP;
-						break;
-					case 0x20:
-						m.command = CMD_PORTAMENTODOWN;
-						break;
-					}
-					m.param = (m.param & 0x0F) | 0xE0;
+				case 0x20:
+					m.command = CMD_PORTAMENTODOWN;
 					break;
 				}
+				m.param = (m.param & 0x0F) | 0xE0;
+				break;
+			default:
+				break;
 			}
+		}
 
-			if (flags & noteFlag) // note + ins
-			{
-				m.instr = chunk.ReadUint8();
-				m.note = chunk.ReadUint8();
-				if(m.note == 0x80) m.note = NOTE_KEYOFF;
-				else if(m.note > 0x80) m.note = NOTE_FADE;	// I guess the support for IT "note fade" notes was not intended in mod2j2b, but hey, it works! :-D
-			}
+		if(flags & noteFlag)  // note + ins
+		{
+			const auto [instr, note] = chunk.ReadArray<uint8, 2>();
+			m.instr = instr;
+			m.note = note;
+			if(m.note == 0x80)
+				m.note = NOTE_KEYOFF;
+			else if(m.note > 0x80)
+				m.note = NOTE_FADE;  // I guess the support for IT "note fade" notes was not intended in mod2j2b, but hey, it works! :-D
+		}
 
-			if (flags & volFlag) // volume
-			{
-				m.volcmd = VOLCMD_VOLUME;
-				m.vol = chunk.ReadUint8();
-				if(isAM)
-				{
-					m.vol = m.vol * 64 / 127;
-				}
-			}
+		if(flags & volFlag) // volume
+		{
+			m.SetVolumeCommand(VOLCMD_VOLUME, chunk.ReadUint8());
+			if(isAM)
+				m.vol = static_cast<ModCommand::VOL>(m.vol * 64u / 127u);
 		}
 	}
 
@@ -713,7 +718,7 @@ bool CSoundFile::ReadAM(FileReader &file, ModLoadingFlags loadFlags)
 
 	FileReader chunkMain(chunks.GetChunk(mainChunkID));
 	AMFFMainChunk mainChunk;
-	if(!chunkMain.IsValid() 
+	if(!chunkMain.IsValid()
 		|| !chunkMain.ReadStruct(mainChunk)
 		|| mainChunk.channels < 1
 		|| !chunkMain.CanRead(mainChunk.channels))
@@ -724,43 +729,38 @@ bool CSoundFile::ReadAM(FileReader &file, ModLoadingFlags loadFlags)
 		return true;
 	}
 
-	InitializeGlobals(MOD_TYPE_J2B);
+	InitializeGlobals(MOD_TYPE_J2B, std::min(static_cast<CHANNELINDEX>(mainChunk.channels), static_cast<CHANNELINDEX>(MAX_BASECHANNELS)));
 	m_SongFlags = SONG_ITOLDEFFECTS | SONG_ITCOMPATGXX;
 	m_SongFlags.set(SONG_LINEARSLIDES, !(mainChunk.flags & AMFFMainChunk::amigaSlides));
 
-	m_nChannels = MIN(mainChunk.channels, MAX_BASECHANNELS);
-	m_nDefaultSpeed = mainChunk.speed;
-	m_nDefaultTempo.Set(mainChunk.tempo);
+	Order().SetDefaultSpeed(mainChunk.speed);
+	Order().SetDefaultTempoInt(mainChunk.tempo);
 	m_nDefaultGlobalVolume = mainChunk.globalvolume * 2;
 
-	m_madeWithTracker = MPT_USTRING("Galaxy Sound System (");
-	if(isAM)
-		m_madeWithTracker += MPT_USTRING("new version)");
-	else
-		m_madeWithTracker += MPT_USTRING("old version)");
-	
-	mpt::String::Read<mpt::String::maybeNullTerminated>(m_songName, mainChunk.songname);
+	m_modFormat.formatName = isAM ? UL_("Galaxy Sound System (new version)") : UL_("Galaxy Sound System (old version)");
+	m_modFormat.type = UL_("j2b");
+	m_modFormat.charset = mpt::Charset::CP437;
+
+	m_songName = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, mainChunk.songname);
 
 	// It seems like there's no way to differentiate between
 	// Muted and Surround channels (they're all 0xA0) - might
 	// be a limitation in mod2j2b.
-	for(CHANNELINDEX nChn = 0; nChn < m_nChannels; nChn++)
+	for(auto &chn : ChnSettings)
 	{
-		ChnSettings[nChn].Reset();
-
 		uint8 pan = chunkMain.ReadUint8();
 		if(isAM)
 		{
 			if(pan > 128)
-				ChnSettings[nChn].dwFlags = CHN_MUTE;
+				chn.dwFlags = CHN_MUTE;
 			else
-				ChnSettings[nChn].nPan = pan * 2;
+				chn.nPan = pan * 2;
 		} else
 		{
 			if(pan >= 128)
-				ChnSettings[nChn].dwFlags = CHN_MUTE;
+				chn.dwFlags = CHN_MUTE;
 			else
-				ChnSettings[nChn].nPan = static_cast<uint16>(std::min(pan * 4, 256));
+				chn.nPan = static_cast<uint16>(std::min(pan * 4, 256));
 		}
 	}
 
@@ -821,7 +821,7 @@ bool CSoundFile::ReadAM(FileReader &file, ModLoadingFlags loadFlags)
 			{
 				AMFFSampleHeader sampleHeader;
 
-				if(m_nSamples + 1 >= MAX_SAMPLES || !chunk.ReadStruct(sampleHeader))
+				if(!CanAddMoreSamples() || !chunk.ReadStruct(sampleHeader))
 				{
 					continue;
 				}
@@ -833,7 +833,7 @@ bool CSoundFile::ReadAM(FileReader &file, ModLoadingFlags loadFlags)
 					continue;
 				}
 
-				mpt::String::Read<mpt::String::maybeNullTerminated>(m_szNames[smp], sampleHeader.name);
+				m_szNames[smp] = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, sampleHeader.name);
 				sampleHeader.ConvertToMPT(instrHeader, Samples[smp]);
 				if(loadFlags & loadSampleData)
 					sampleHeader.GetSampleFormat().ReadSample(Samples[smp], chunk);
@@ -883,7 +883,7 @@ bool CSoundFile::ReadAM(FileReader &file, ModLoadingFlags loadFlags)
 
 			for(auto sampleChunk : sampleChunks)
 			{
-				if(sampleChunk.ReadUint32LE() != AMFFRiffChunk::idAS__ || m_nSamples + 1 >= MAX_SAMPLES)
+				if(sampleChunk.ReadUint32LE() != AMFFRiffChunk::idAS__ || !CanAddMoreSamples())
 				{
 					continue;
 				}
@@ -911,7 +911,7 @@ bool CSoundFile::ReadAM(FileReader &file, ModLoadingFlags loadFlags)
 					break;
 				}
 
-				mpt::String::Read<mpt::String::maybeNullTerminated>(m_szNames[smp], sampleHeader.name);
+				m_szNames[smp] = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, sampleHeader.name);
 
 				sampleHeader.ConvertToMPT(instrHeader, Samples[smp]);
 
@@ -1015,23 +1015,76 @@ bool CSoundFile::ReadJ2B(FileReader &file, ModLoadingFlags loadFlags)
 		return true;
 	}
 
-	FileReader::PinnedRawDataView filePackedView = file.GetPinnedRawDataView(fileHeader.packedLength);
-
-#ifndef MPT_BUILD_FUZZER
-	if(fileHeader.crc32 != crc32(0, mpt::byte_cast<const Bytef*>(filePackedView.data()), static_cast<uint32>(filePackedView.size())))
+	// Header is valid, now unpack the RIFF AM file using inflate
+	z_stream strm{};
+	int zlib_errc = Z_OK;
+	zlib_errc = inflateInit(&strm);
+	if(zlib_errc == Z_MEM_ERROR)
+	{
+		mpt::throw_out_of_memory();
+	} else if(zlib_errc < Z_OK)
 	{
 		return false;
 	}
-#endif
 
-	// Header is valid, now unpack the RIFF AM file using inflate
-	uLongf destSize = fileHeader.unpackedLength;
-	std::vector<Bytef> amFileData(destSize);
-	int retVal = uncompress(amFileData.data(), &destSize, mpt::byte_cast<const Bytef*>(filePackedView.data()), static_cast<uint32>(filePackedView.size()));
+	uint32 remainRead = fileHeader.packedLength, remainWrite = fileHeader.unpackedLength, totalWritten = 0;
+	uint32 crc = 0;
+	std::vector<Bytef> amFileData;
+	int retVal = Z_OK;
+	try
+	{
+		amFileData.resize(remainWrite);
+		while(remainRead && remainWrite && retVal != Z_STREAM_END)
+		{
+			Bytef buffer[mpt::IO::BUFFERSIZE_TINY];
+			uint32 readSize = std::min(static_cast<uint32>(sizeof(buffer)), remainRead);
+			file.ReadRaw(mpt::span(buffer, readSize));
+			crc = static_cast<uint32>(crc32(crc, buffer, readSize));
+
+			strm.avail_in = readSize;
+			strm.next_in = buffer;
+			do
+			{
+				strm.avail_out = remainWrite;
+				strm.next_out = amFileData.data() + totalWritten;
+				zlib_errc = inflate(&strm, Z_NO_FLUSH);
+				if(zlib_errc == Z_BUF_ERROR)
+				{
+					// expected
+				} else if(zlib_errc == Z_MEM_ERROR)
+				{
+					mpt::throw_out_of_memory();
+				} else if(zlib_errc < Z_OK)
+				{
+					inflateEnd(&strm);
+					return false;
+				}
+				retVal = zlib_errc;
+				uint32 written = remainWrite - strm.avail_out;
+				totalWritten += written;
+				remainWrite -= written;
+			} while(remainWrite && strm.avail_out == 0);
+
+			remainRead -= readSize;
+		}
+	} catch(mpt::out_of_memory e)
+	{
+		inflateEnd(&strm);
+		mpt::rethrow_out_of_memory(e);	
+	} catch(const std::exception &)
+	{
+		inflateEnd(&strm);
+		return false;
+	} catch(...)
+	{
+		inflateEnd(&strm);
+		return false;
+	}
+	inflateEnd(&strm);
 
 	bool result = false;
 #ifndef MPT_BUILD_FUZZER
-	if(destSize == fileHeader.unpackedLength && retVal == Z_OK)
+	if(fileHeader.crc32 == crc && !remainWrite && retVal == Z_STREAM_END)
 #endif
 	{
 		// Success, now load the RIFF AM(FF) module.
