@@ -9,39 +9,43 @@
 
 
 #include "stdafx.h"
-#include "Sndfile.h"
+#include "modcommand.h"
 #include "mod_specifications.h"
+#include "Sndfile.h"
 #include "Tables.h"
 
 
 OPENMPT_NAMESPACE_BEGIN
 
 
-const EffectType effectTypes[] =
+static constexpr EffectType effectTypes[] =
 {
-	EFFECT_TYPE_NORMAL, EFFECT_TYPE_NORMAL,  EFFECT_TYPE_PITCH,  EFFECT_TYPE_PITCH,
-	EFFECT_TYPE_PITCH,  EFFECT_TYPE_PITCH,   EFFECT_TYPE_VOLUME, EFFECT_TYPE_VOLUME,
-	EFFECT_TYPE_VOLUME, EFFECT_TYPE_PANNING, EFFECT_TYPE_NORMAL, EFFECT_TYPE_VOLUME,
-	EFFECT_TYPE_GLOBAL, EFFECT_TYPE_VOLUME,  EFFECT_TYPE_GLOBAL, EFFECT_TYPE_NORMAL,
-	EFFECT_TYPE_GLOBAL, EFFECT_TYPE_GLOBAL,  EFFECT_TYPE_NORMAL, EFFECT_TYPE_NORMAL,
-	EFFECT_TYPE_NORMAL, EFFECT_TYPE_VOLUME,  EFFECT_TYPE_VOLUME, EFFECT_TYPE_GLOBAL,
-	EFFECT_TYPE_GLOBAL, EFFECT_TYPE_NORMAL,  EFFECT_TYPE_PITCH,  EFFECT_TYPE_PANNING,
-	EFFECT_TYPE_PITCH,  EFFECT_TYPE_PANNING, EFFECT_TYPE_NORMAL, EFFECT_TYPE_NORMAL,
-	EFFECT_TYPE_NORMAL, EFFECT_TYPE_NORMAL,  EFFECT_TYPE_NORMAL, EFFECT_TYPE_PITCH,
-	EFFECT_TYPE_PITCH,  EFFECT_TYPE_NORMAL,  EFFECT_TYPE_PITCH,  EFFECT_TYPE_PITCH,
-	EFFECT_TYPE_PITCH,  EFFECT_TYPE_PITCH,   EFFECT_TYPE_NORMAL, EFFECT_TYPE_NORMAL,
-	EFFECT_TYPE_NORMAL, EFFECT_TYPE_NORMAL,
+	EffectType::Normal, EffectType::Normal,  EffectType::Pitch,  EffectType::Pitch,
+	EffectType::Pitch,  EffectType::Pitch,   EffectType::Volume, EffectType::Volume,
+	EffectType::Volume, EffectType::Panning, EffectType::Normal, EffectType::Volume,
+	EffectType::Global, EffectType::Volume,  EffectType::Global, EffectType::Normal,
+	EffectType::Global, EffectType::Global,  EffectType::Normal, EffectType::Normal,
+	EffectType::Normal, EffectType::Volume,  EffectType::Volume, EffectType::Global,
+	EffectType::Global, EffectType::Normal,  EffectType::Pitch,  EffectType::Panning,
+	EffectType::Pitch,  EffectType::Panning, EffectType::Normal, EffectType::Normal,
+	EffectType::Normal, EffectType::Normal,  EffectType::Normal, EffectType::Pitch,
+	EffectType::Pitch,  EffectType::Normal,  EffectType::Pitch,  EffectType::Pitch,
+	EffectType::Pitch,  EffectType::Pitch,   EffectType::Normal, EffectType::Normal,
+	EffectType::Normal, EffectType::Normal,  EffectType::Volume, EffectType::Normal,
+	EffectType::Normal, EffectType::Volume,  EffectType::Pitch,  EffectType::Pitch,
+	EffectType::Pitch,  EffectType::Pitch,   EffectType::Pitch,  EffectType::Pitch,
+	EffectType::Volume, EffectType::Volume,
 };
 
 static_assert(std::size(effectTypes) == MAX_EFFECTS);
 
 
-const EffectType volumeEffectTypes[] =
+static constexpr EffectType volumeEffectTypes[] =
 {
-	EFFECT_TYPE_NORMAL, EFFECT_TYPE_VOLUME,  EFFECT_TYPE_PANNING, EFFECT_TYPE_VOLUME,
-	EFFECT_TYPE_VOLUME, EFFECT_TYPE_VOLUME,  EFFECT_TYPE_VOLUME,  EFFECT_TYPE_PITCH,
-	EFFECT_TYPE_PITCH,  EFFECT_TYPE_PANNING, EFFECT_TYPE_PANNING, EFFECT_TYPE_PITCH,
-	EFFECT_TYPE_PITCH,  EFFECT_TYPE_PITCH,   EFFECT_TYPE_NORMAL,  EFFECT_TYPE_NORMAL,
+	EffectType::Normal, EffectType::Volume,  EffectType::Panning, EffectType::Volume,
+	EffectType::Volume, EffectType::Volume,  EffectType::Volume,  EffectType::Pitch,
+	EffectType::Pitch,  EffectType::Panning, EffectType::Panning, EffectType::Pitch,
+	EffectType::Pitch,  EffectType::Pitch,   EffectType::Normal,  EffectType::Normal,
 };
 
 static_assert(std::size(volumeEffectTypes) == MAX_VOLCMDS);
@@ -52,7 +56,7 @@ EffectType ModCommand::GetEffectType(COMMAND cmd)
 	if(cmd < std::size(effectTypes))
 		return effectTypes[cmd];
 	else
-		return EFFECT_TYPE_NORMAL;
+		return EffectType::Normal;
 }
 
 
@@ -61,7 +65,7 @@ EffectType ModCommand::GetVolumeEffectType(VOLCMD volcmd)
 	if(volcmd < std::size(volumeEffectTypes))
 		return volumeEffectTypes[volcmd];
 	else
-		return EFFECT_TYPE_NORMAL;
+		return EffectType::Normal;
 }
 
 
@@ -147,6 +151,11 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 		command = CMD_S3MCMDEX;
 		param = 0x9F;
 	}
+	if(command == CMD_VOLUME8)
+	{
+		command = CMD_VOLUME;
+		param = static_cast<PARAM>((param + 3u) / 4u);
+	}
 
 	// helper variables
 	const bool oldTypeIsMOD = (fromType == MOD_TYPE_MOD), oldTypeIsXM = (fromType == MOD_TYPE_XM),
@@ -169,13 +178,13 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 	{
 		if(newTypeIsS3M)
 		{
-			param = (param + 1) >> 1;
+			param = static_cast<uint8>((param + 1u) >> 1);
 		} else if(oldTypeIsS3M)
 		{
 			if(param == 0xA4)
 			{
 				// surround remap
-				command = static_cast<COMMAND>((toType & (MOD_TYPE_IT | MOD_TYPE_MPT)) ? CMD_S3MCMDEX : CMD_XFINEPORTAUPDOWN);
+				command = (toType & (MOD_TYPE_IT | MOD_TYPE_MPT)) ? CMD_S3MCMDEX : CMD_XFINEPORTAUPDOWN;
 				param = 0x91;
 			} else
 			{
@@ -196,7 +205,7 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 	{
 		if(IsPcNote())
 		{
-			COMMAND newCmd = static_cast<COMMAND>(note == NOTE_PC ? CMD_MIDI : CMD_SMOOTHMIDI);
+			COMMAND newCmd = (note == NOTE_PC) ? CMD_MIDI : CMD_SMOOTHMIDI;
 			if(!newSpecs.HasCommand(newCmd))
 			{
 				newCmd = CMD_MIDI;	// assuming that this was CMD_SMOOTHMIDI
@@ -234,15 +243,15 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 				param -= 0x80;
 			}
 			if(param <= 30)
-				param = 0xE0 | ((param + 1u) / 2u);
+				param = static_cast<uint8>(0xE0u | ((param + 1u) / 2u));
 			else
-				param = 0xF0 | std::min(static_cast<PARAM>((param + 7u) / 8u), PARAM(15));
+				param = static_cast<uint8>(0xF0u | std::min(static_cast<PARAM>((param + 7u) / 8u), PARAM(15)));
 		}
 	} // End if(oldTypeIsMPT)
 
 	/////////////////////////////////////////
 	// Convert MOD / XM to S3M / IT / MPTM
-	if(oldTypeIsMOD_XM && newTypeIsS3M_IT_MPT)
+	if(!oldTypeIsS3M_IT_MPT && newTypeIsS3M_IT_MPT)
 	{
 		switch(command)
 		{
@@ -309,7 +318,7 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 				command = CMD_S3MCMDEX;
 				if(param == 0)
 					instr = 0;
-				param = 0xD0 | (param & 0x0F);
+				param = 0xD0 | std::min(param, PARAM(0x0F));
 			}
 			break;
 
@@ -324,15 +333,21 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 			}
 			break;
 
+		case CMD_DUMMY:
+			// Command "W" in XM, equivalent implementation in IT / S3M is command " ".
+			// Don't convert this command, as it overwrites effect memory in S3M in particular - a feature not supported by all S3M players.
+			command = CMD_NONE;
+			break;
+
 		default:
 			break;
 		}
-	} // End if(oldTypeIsMOD_XM && newTypeIsS3M_IT_MPT)
+	} // End if(!oldTypeIsS3M_IT_MPT && newTypeIsS3M_IT_MPT)
 
 
 	/////////////////////////////////////////
 	// Convert S3M / IT / MPTM to MOD / XM
-	else if(oldTypeIsS3M_IT_MPT && newTypeIsMOD_XM)
+	else if(!oldTypeIsMOD_XM && newTypeIsMOD_XM)
 	{
 		if(note == NOTE_NOTECUT)
 		{
@@ -398,7 +413,7 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 				} else
 				{
 					command = CMD_MODCMDEX;
-					param = (((param & 0x0F) + 3) >> 2) | 0x10;
+					param = static_cast<uint8>((((param & 0x0Fu) + 3u) >> 2) | 0x10u);
 				}
 			} else
 			{
@@ -420,7 +435,7 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 				} else
 				{
 					command = CMD_MODCMDEX;
-					param = (((param & 0x0F) + 3) >> 2) | 0x20;
+					param = static_cast<uint8>((((param & 0x0Fu) + 3u) >> 2) | 0x20u);
 				}
 			} else
 			{
@@ -463,7 +478,7 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 		default:
 			break;
 		}
-	} // End if(oldTypeIsS3M_IT_MPT && newTypeIsMOD_XM)
+	} // End if(!oldTypeIsMOD_XM && newTypeIsMOD_XM)
 
 
 	///////////////////////
@@ -495,7 +510,7 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 			break;
 
 		case CMD_GLOBALVOLUME:
-			param = (std::min(PARAM(0x80), param) + 1) / 2u;
+			param = static_cast<uint8>((std::min(PARAM(0x80), param) + 1u) / 2u);
 			break;
 
 		default:
@@ -511,10 +526,12 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 		{
 		case CMD_VIBRATO:
 			// With linear slides, strength is roughly doubled.
-			param = (param & 0xF0) | (((param & 0x0F) + 1) / 2u);
+			param = static_cast<uint8>((param & 0xF0u) | (((param & 0x0Fu) + 1u) / 2u));
 			break;
 		case CMD_GLOBALVOLUME:
-			param = (std::min(PARAM(0x80), param) + 1) / 2u;
+			param = static_cast<uint8>((std::min(PARAM(0x80), param) + 1u) / 2u);
+			break;
+		default:
 			break;
 		}
 	} // End if(oldTypeIsIT_MPT && newTypeIsXM)
@@ -532,6 +549,8 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 		case CMD_GLOBALVOLUME:
 			param = std::min(PARAM(0x40), param) * 2u;
 			break;
+		default:
+			break;
 		}
 	} // End if(oldTypeIsIT_MPT && newTypeIsXM)
 
@@ -548,6 +567,8 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 		case CMD_TEMPO:
 			param = std::max(param, PARAM(0x20));
 			break;
+		default:
+			break;
 		}
 	}
 
@@ -587,6 +608,8 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 			if((param & 0xF0) == 0xF0)
 				command = CMD_NONE;
 			break;
+		default:
+			break;
 		}
 	} // End if(oldTypeIsMOD && newTypeIsXM)
 
@@ -620,6 +643,9 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 			case CMD_MODCMDEX: // This would turn into "Invert Loop", so let's better remove it
 				if((param & 0xF0) == 0xF0) command = CMD_NONE;
 				break;
+
+			default:
+				break;
 		}
 
 		if(command == CMD_NONE) switch(volcmd)
@@ -678,6 +704,9 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 				command = CMD_VIBRATO;
 				param = vol << 4;
 				break;
+
+			default:
+				break;
 		}
 		volcmd = VOLCMD_NONE;
 	} // End if(newTypeIsMOD)
@@ -753,6 +782,9 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 				param = vol;
 				volcmd = VOLCMD_NONE;
 				break;
+
+			default:
+				break;
 		}
 	} // End if(newTypeIsS3M)
 
@@ -801,6 +833,9 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 				param = ImpulseTrackerPortaVolCmd[vol & 0x0F];
 				volcmd = VOLCMD_NONE;
 				break;
+
+			default:
+				break;
 		}
 	} // End if(newTypeIsXM)
 
@@ -834,6 +869,9 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 				param = vol << 4;
 				volcmd = VOLCMD_NONE;
 				break;
+
+			default:
+				break;
 		}
 
 		switch(volcmd)
@@ -850,6 +888,8 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 		case VOLCMD_OFFSET:
 			vol = std::min(vol, VOL(9));
 			break;
+		default:
+			break;
 		}
 	} // End if(newTypeIsIT_MPT)
 
@@ -874,11 +914,29 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 		{
 			param = vol << 3;
 		}
+	} else if(volcmd == VOLCMD_PLAYCONTROL && (vol == 2 || vol == 3) && command == CMD_NONE
+		&& !newSpecs.HasVolCommand(VOLCMD_PLAYCONTROL)
+		&& (newSpecs.HasCommand(CMD_S3MCMDEX) || newSpecs.HasCommand(CMD_XFINEPORTAUPDOWN)))
+	{
+		volcmd = VOLCMD_NONE;
+		param = vol - 2 + 0x9E;
+		if(newSpecs.HasCommand(CMD_S3MCMDEX))
+			command = CMD_S3MCMDEX;
+		else
+			command = CMD_XFINEPORTAUPDOWN;
 	}
 
+	// Offset effect memory is only updated when the command is placed next to a note.
+	if(oldTypeIsXM && command == CMD_OFFSET && !IsNote())
+		command = CMD_NONE;
+
 	if((command == CMD_REVERSEOFFSET || command == CMD_OFFSETPERCENTAGE) && !newSpecs.HasCommand(command))
 	{
 		command = CMD_OFFSET;
+	} else if(command == CMD_HMN_MEGA_ARP && !newSpecs.HasCommand(CMD_HMN_MEGA_ARP))
+	{
+		command = CMD_ARPEGGIO;
+		param = (HisMastersNoiseMegaArp[param & 0x0F][1] << 4) | HisMastersNoiseMegaArp[param & 0x0F][2];
 	}
 
 	if(!newSpecs.HasNote(note))
@@ -893,6 +951,46 @@ void ModCommand::Convert(MODTYPE fromType, MODTYPE toType, const CSoundFile &snd
 }
 
 
+bool ModCommand::IsAnyPitchSlide() const
+{
+	switch(command)
+	{
+	case CMD_PORTAMENTOUP:
+	case CMD_PORTAMENTODOWN:
+	case CMD_TONEPORTAMENTO:
+	case CMD_TONEPORTAVOL:
+	case CMD_NOTESLIDEUP:
+	case CMD_NOTESLIDEDOWN:
+	case CMD_NOTESLIDEUPRETRIG:
+	case CMD_NOTESLIDEDOWNRETRIG:
+	case CMD_AUTO_PORTAUP:
+	case CMD_AUTO_PORTADOWN:
+	case CMD_AUTO_PORTAUP_FINE:
+	case CMD_AUTO_PORTADOWN_FINE:
+	case CMD_AUTO_PORTAMENTO_FC:
+	case CMD_TONEPORTA_DURATION:
+		return true;
+	case CMD_MODCMDEX:
+	case CMD_XFINEPORTAUPDOWN:
+		if(param >= 0x10 && param <= 0x2F)
+			return true;
+		break;
+	default:
+		break;
+	}
+	switch(volcmd)
+	{
+	case VOLCMD_TONEPORTAMENTO:
+	case VOLCMD_PORTAUP:
+	case VOLCMD_PORTADOWN:
+		return true;
+	default:
+		break;
+	}
+	return false;
+}
+
+
 bool ModCommand::IsContinousCommand(const CSoundFile &sndFile) const
 {
 	switch(command)
@@ -910,6 +1008,13 @@ bool ModCommand::IsContinousCommand(const CSoundFile &sndFile) const
 	case CMD_NOTESLIDEDOWN:
 	case CMD_NOTESLIDEUPRETRIG:
 	case CMD_NOTESLIDEDOWNRETRIG:
+	case CMD_HMN_MEGA_ARP:
+	case CMD_AUTO_VOLUMESLIDE:
+	case CMD_AUTO_PORTAUP:
+	case CMD_AUTO_PORTADOWN:
+	case CMD_AUTO_PORTAUP_FINE:
+	case CMD_AUTO_PORTADOWN_FINE:
+	case CMD_AUTO_PORTAMENTO_FC:
 		return true;
 	case CMD_PORTAMENTOUP:
 	case CMD_PORTAMENTODOWN:
@@ -975,6 +1080,7 @@ bool ModCommand::IsSlideUpDownCommand() const
 		case CMD_GLOBALVOLSLIDE:
 		case CMD_CHANNELVOLSLIDE:
 		case CMD_PANNINGSLIDE:
+		case CMD_AUTO_VOLUMESLIDE:
 			return true;
 		default:
 			return false;
@@ -1024,6 +1130,40 @@ bool ModCommand::IsGlobalCommand(COMMAND command, PARAM param)
 	}
 }
 
+
+bool ModCommand::CommandHasTwoNibbles(COMMAND command)
+{
+	switch(command)
+	{
+	case CMD_ARPEGGIO:
+	case CMD_VIBRATO:
+	case CMD_TONEPORTAVOL:
+	case CMD_VIBRATOVOL:
+	case CMD_TREMOLO:
+	case CMD_VOLUMESLIDE:
+	case CMD_RETRIG:
+	case CMD_TREMOR:
+	case CMD_MODCMDEX:
+	case CMD_S3MCMDEX:
+	case CMD_CHANNELVOLSLIDE:
+	case CMD_GLOBALVOLSLIDE:
+	case CMD_FINEVIBRATO:
+	case CMD_PANBRELLO:
+	case CMD_XFINEPORTAUPDOWN:
+	case CMD_PANNINGSLIDE:
+	case CMD_DELAYCUT:
+	case CMD_NOTESLIDEUP:
+	case CMD_NOTESLIDEDOWN:
+	case CMD_NOTESLIDEUPRETRIG:
+	case CMD_NOTESLIDEDOWNRETRIG:
+	case CMD_AUTO_VOLUMESLIDE:
+		return true;
+	default:
+		return false;
+	}
+}
+
+
 // "Importance" of every FX command. Table is used for importing from formats with multiple effect colums
 // and is approximately the same as in SchismTracker.
 size_t ModCommand::GetEffectWeight(COMMAND cmd)
@@ -1035,6 +1175,7 @@ size_t ModCommand::GetEffectWeight(COMMAND cmd)
 		CMD_DUMMY,
 		CMD_XPARAM,
 		CMD_SETENVPOSITION,
+		CMD_MED_SYNTH_JUMP,
 		CMD_KEYOFF,
 		CMD_TREMOLO,
 		CMD_FINEVIBRATO,
@@ -1055,21 +1196,32 @@ size_t ModCommand::GetEffectWeight(COMMAND cmd)
 		CMD_NOTESLIDEDOWNRETRIG,
 		CMD_NOTESLIDEDOWN,
 		CMD_PORTAMENTOUP,
+		CMD_AUTO_PORTAMENTO_FC,
+		CMD_AUTO_PORTAUP_FINE,
+		CMD_AUTO_PORTAUP,
 		CMD_PORTAMENTODOWN,
+		CMD_AUTO_PORTADOWN_FINE,
+		CMD_AUTO_PORTADOWN,
 		CMD_VOLUMESLIDE,
+		CMD_AUTO_VOLUMESLIDE,
 		CMD_VIBRATOVOL,
 		CMD_VOLUME,
+		CMD_VOLUME8,
 		CMD_DIGIREVERSESAMPLE,
 		CMD_REVERSEOFFSET,
 		CMD_OFFSETPERCENTAGE,
 		CMD_OFFSET,
 		CMD_TREMOR,
 		CMD_RETRIG,
+		CMD_HMN_MEGA_ARP,
 		CMD_ARPEGGIO,
+		CMD_TONEPORTA_DURATION,
 		CMD_TONEPORTAMENTO,
 		CMD_TONEPORTAVOL,
 		CMD_DBMECHO,
-		CMD_GLOBALVOLSLIDE,
+		CMD_VOLUMEDOWN_DURATION,
+		CMD_VOLUMEDOWN_ETX,
+		CMD_CHANNELVOLSLIDE,
 		CMD_CHANNELVOLUME,
 		CMD_GLOBALVOLSLIDE,
 		CMD_GLOBALVOLUME,
@@ -1088,47 +1240,43 @@ size_t ModCommand::GetEffectWeight(COMMAND cmd)
 		}
 	}
 	// Invalid / unknown command.
+	MPT_ASSERT_NOTREACHED();
 	return 0;
 }
 
 
-// Try to convert a fx column command (&effect) into a volume column command.
-// Returns true if successful.
+// Try to convert a fx column command into a volume column command.
+// Returns the translated command if successful.
 // Some commands can only be converted by losing some precision.
 // If moving the command into the volume column is more important than accuracy, use force = true.
 // (Code translated from SchismTracker and mainly supposed to be used with loaders ported from this tracker)
-bool ModCommand::ConvertVolEffect(uint8 &effect, uint8 &param, bool force)
+std::pair<VolumeCommand, ModCommand::VOL> ModCommand::ConvertToVolCommand(const EffectCommand effect, ModCommand::PARAM param, bool force)
 {
 	switch(effect)
 	{
 	case CMD_NONE:
-		effect = VOLCMD_NONE;
-		return true;
-	case CMD_VOLUME:
-		effect = VOLCMD_VOLUME;
-		param = std::min(param, PARAM(64));
 		break;
+	case CMD_VOLUME:
+		return {VOLCMD_VOLUME, std::min(param, PARAM(64))};
+	case CMD_VOLUME8:
+		if(!force && (param & 3))
+			break;
+		return {VOLCMD_VOLUME, static_cast<VOL>((param + 3u) / 4u)};
 	case CMD_PORTAMENTOUP:
 		// if not force, reject when dividing causes loss of data in LSB, or if the final value is too
 		// large to fit. (volume column Ex/Fx are four times stronger than effect column)
 		if(!force && ((param & 3) || param >= 0xE0))
-			return false;
-		param /= 4;
-		effect = VOLCMD_PORTAUP;
-		break;
+			break;
+		return {VOLCMD_PORTAUP, static_cast<VOL>(param / 4u)};
 	case CMD_PORTAMENTODOWN:
 		if(!force && ((param & 3) || param >= 0xE0))
-			return false;
-		param /= 4;
-		effect = VOLCMD_PORTADOWN;
-		break;
+			break;
+		return {VOLCMD_PORTADOWN, static_cast<VOL>(param / 4u)};
 	case CMD_TONEPORTAMENTO:
 		if(param >= 0xF0)
 		{
 			// hack for people who can't type F twice :)
-			effect = VOLCMD_TONEPORTAMENTO;
-			param = 9;
-			return true;
+			return {VOLCMD_TONEPORTAMENTO, VOL(9)};
 		}
 		for(uint8 n = 0; n < 10; n++)
 		{
@@ -1136,83 +1284,74 @@ bool ModCommand::ConvertVolEffect(uint8 &effect, uint8 &param, bool force)
 				? (param <= ImpulseTrackerPortaVolCmd[n])
 				: (param == ImpulseTrackerPortaVolCmd[n]))
 			{
-				effect = VOLCMD_TONEPORTAMENTO;
-				param = n;
-				return true;
+				return {VOLCMD_TONEPORTAMENTO, n};
 			}
 		}
-		return false;
+		break;
 	case CMD_VIBRATO:
 		if(force)
 			param = std::min(static_cast<PARAM>(param & 0x0F), PARAM(9));
 		else if((param & 0x0F) > 9 || (param & 0xF0) != 0)
-			return false;
-		param &= 0x0F;
-		effect = VOLCMD_VIBRATODEPTH;
-		break;
+			break;
+		return {VOLCMD_VIBRATODEPTH, static_cast<VOL>(param & 0x0F)};
 	case CMD_FINEVIBRATO:
 		if(force)
 			param = 0;
 		else if(param)
-			return false;
-		effect = VOLCMD_VIBRATODEPTH;
-		break;
+			break;
+		return {VOLCMD_VIBRATODEPTH, param};
 	case CMD_PANNING8:
 		if(param == 255)
 			param = 64;
 		else
 			param /= 4;
-		effect = VOLCMD_PANNING;
-		break;
+		return {VOLCMD_PANNING, param};
 	case CMD_VOLUMESLIDE:
 		if(param == 0)
-			return false;
-		if((param & 0xF) == 0)	// Dx0 / Cx
-		{
-			param >>= 4;
-			effect = VOLCMD_VOLSLIDEUP;
-		} else if((param & 0xF0) == 0)	// D0x / Dx
-		{
-			effect = VOLCMD_VOLSLIDEDOWN;
-		} else if((param & 0xF) == 0xF)	// DxF / Ax
-		{
-			param >>= 4;
-			effect = VOLCMD_FINEVOLUP;
-		} else if((param & 0xF0) == 0xF0)	// DFx / Bx
-		{
-			param &= 0xF;
-			effect = VOLCMD_FINEVOLDOWN;
-		} else // ???
-		{
-			return false;
-		}
+			break;
+		if((param & 0x0F) == 0)  // Dx0 / Cx
+			return {VOLCMD_VOLSLIDEUP, static_cast<VOL>(param >> 4)};
+		else if((param & 0xF0) == 0)  // D0x / Dx
+			return {VOLCMD_VOLSLIDEDOWN, param};
+		else if((param & 0x0F) == 0x0F)  // DxF / Ax
+			return {VOLCMD_FINEVOLUP, static_cast<VOL>(param >> 4)};
+		else if((param & 0xF0) == 0xF0)  // DFx / Bx
+			return {VOLCMD_FINEVOLDOWN, static_cast<VOL>(param & 0x0F)};
 		break;
 	case CMD_S3MCMDEX:
-		switch (param >> 4)
+		switch(param & 0xF0)
 		{
-		case 8:
-			effect = VOLCMD_PANNING;
-			param = ((param & 0xF) << 2) + 2;
-			return true;
-		case 0: case 1: case 2: case 0xF:
-			if(force)
-			{
-				effect = param = 0;
-				return true;
-			}
+		case 0x80:
+			return {VOLCMD_PANNING, static_cast<VOL>(((param & 0x0F) << 2) + 2)};
+		case 0x90:
+			if(param >= 0x9E && force)
+				return {VOLCMD_PLAYCONTROL, static_cast<VOL>(param - 0x9E + 2)};
 			break;
 		default:
 			break;
 		}
-		return false;
+		break;
+	case CMD_MODCMDEX:
+		switch(param & 0xF0)
+		{
+			case 0x80:
+				return {VOLCMD_PANNING, static_cast<VOL>(((param & 0x0F) << 2) + 2)};
+			case 0xA0:
+				return {VOLCMD_FINEVOLUP, static_cast<VOL>(param & 0x0F)};
+			case 0xB0:
+				return {VOLCMD_FINEVOLDOWN, static_cast<VOL>(param & 0x0F)};
+			default:
+				break;
+		}
+		break;
 	default:
-		return false;
+		break;
 	}
-	return true;
+	return {VOLCMD_NONE, VOL(0)};
 }
 
 // Try to combine two commands into one. Returns true on success and the combined command is placed in eff1 / param1.
-bool ModCommand::CombineEffects(uint8 &eff1, uint8 &param1, uint8 &eff2, uint8 &param2)
+bool ModCommand::CombineEffects(EffectCommand &eff1, uint8 &param1, EffectCommand &eff2, uint8 &param2)
 {
 	if(eff1 == CMD_VOLUMESLIDE && (eff2 == CMD_VIBRATO || eff2 == CMD_TONEPORTAVOL) && param2 == 0)
 	{
@@ -1259,12 +1398,51 @@ bool ModCommand::CombineEffects(uint8 &eff1, uint8 &param1, uint8 &eff2, uint8 &
 }
 
 
-std::pair<EffectCommand, ModCommand::PARAM> ModCommand::TwoRegularCommandsToMPT(uint8 &effect1, uint8 &param1, uint8 &effect2, uint8 &param2)
+std::pair<EffectCommand, ModCommand::PARAM> ModCommand::FillInTwoCommands(EffectCommand effect1, uint8 param1, EffectCommand effect2, uint8 param2, bool allowLowResOffset)
 {
+	if(effect1 == effect2)
+	{
+		// For non-sliding, absolute effects, it doesn't make sense to keep both commands
+		switch(effect1)
+		{
+		case CMD_ARPEGGIO:
+		case CMD_PANNING8:
+		case CMD_OFFSET:
+		case CMD_POSITIONJUMP:
+		case CMD_VOLUME:
+		case CMD_PATTERNBREAK:
+		case CMD_SPEED:
+		case CMD_TEMPO:
+		case CMD_CHANNELVOLUME:
+		case CMD_GLOBALVOLUME:
+		case CMD_KEYOFF:
+		case CMD_SETENVPOSITION:
+		case CMD_MIDI:
+		case CMD_SMOOTHMIDI:
+		case CMD_DELAYCUT:
+		case CMD_FINETUNE:
+		case CMD_FINETUNE_SMOOTH:
+		case CMD_DUMMY:
+		case CMD_REVERSEOFFSET:
+		case CMD_DBMECHO:
+		case CMD_OFFSETPERCENTAGE:
+		case CMD_DIGIREVERSESAMPLE:
+		case CMD_VOLUME8:
+		case CMD_HMN_MEGA_ARP:
+		case CMD_MED_SYNTH_JUMP:
+			effect2 = CMD_NONE;
+			break;
+		default:
+			break;
+		}
+	}
+
 	for(uint8 n = 0; n < 4; n++)
 	{
-		if(ModCommand::ConvertVolEffect(effect1, param1, (n > 1)))
+		if(auto volCmd = ModCommand::ConvertToVolCommand(effect1, param1, (n > 1)); effect1 == CMD_NONE || volCmd.first != VOLCMD_NONE)
 		{
+			SetVolumeCommand(volCmd);
+			SetEffectCommand(effect2, param2);
 			return {CMD_NONE, ModCommand::PARAM(0)};
 		}
 		std::swap(effect1, effect2);
@@ -1272,15 +1450,20 @@ std::pair<EffectCommand, ModCommand::PARAM> ModCommand::TwoRegularCommandsToMPT(
 	}
 
 	// Can only keep one command :(
-	if(GetEffectWeight(static_cast<COMMAND>(effect1)) > GetEffectWeight(static_cast<COMMAND>(effect2)))
+	if(GetEffectWeight(effect1) > GetEffectWeight(effect2))
 	{
 		std::swap(effect1, effect2);
 		std::swap(param1, param2);
 	}
-	std::pair<EffectCommand, PARAM> lostCommand = {static_cast<EffectCommand>(effect1), param1};
-	effect1 = VOLCMD_NONE;
-	param1 = 0;
-	return lostCommand;
+	if(effect2 == CMD_OFFSET && (allowLowResOffset || param2 == 0))
+	{
+		SetVolumeCommand(VOLCMD_OFFSET, static_cast<ModCommand::VOL>(param2 ? std::max(param2 * 9 / 255, 1) : 0));
+		SetEffectCommand(effect1, param1);
+		return {CMD_NONE, ModCommand::PARAM(0)};
+	}
+	SetVolumeCommand(VOLCMD_NONE, 0);
+	SetEffectCommand(effect2, param2);
+	return {effect1, param1};
 }
 
 
