@@ -16,56 +16,70 @@
 #include "ITTools.h"
 #include "Sndfile.h"
 #include "mod_specifications.h"
+#include "mpt/io/io.hpp"
+#include "mpt/io/io_stdstream.hpp"
 
 
 OPENMPT_NAMESPACE_BEGIN
 
 
-CSoundFile& CPattern::GetSoundFile() { return m_rPatternContainer.GetSoundFile(); }
-const CSoundFile& CPattern::GetSoundFile() const { return m_rPatternContainer.GetSoundFile(); }
+CSoundFile& CPattern::GetSoundFile() noexcept { return m_rPatternContainer.GetSoundFile(); }
+const CSoundFile& CPattern::GetSoundFile() const noexcept { return m_rPatternContainer.GetSoundFile(); }
 
 
-CHANNELINDEX CPattern::GetNumChannels() const
+CHANNELINDEX CPattern::GetNumChannels() const noexcept
 {
 	return GetSoundFile().GetNumChannels();
 }
 
 
 // Check if there is any note data on a given row.
-bool CPattern::IsEmptyRow(ROWINDEX row) const
+bool CPattern::IsEmptyRow(ROWINDEX row) const noexcept
 {
 	if(m_ModCommands.empty() || !IsValidRow(row))
-	{
 		return true;
-	}
 
-	PatternRow data = GetRow(row);
-	for(CHANNELINDEX chn = 0; chn < GetNumChannels(); chn++, data++)
+	for(const auto &m : GetRow(row))
 	{
-		if(!data->IsEmpty())
-		{
+		if(!m.IsEmpty())
 			return false;
-		}
 	}
 	return true;
 }
 
 
-bool CPattern::SetSignature(const ROWINDEX rowsPerBeat, const ROWINDEX rowsPerMeasure)
+// Check if the row contains any position jumps or pattern breaks.
+bool CPattern::RowHasJump(ROWINDEX row) const noexcept
 {
-	if(rowsPerBeat < 1
-		|| rowsPerBeat > GetSoundFile().GetModSpecifications().patternRowsMax
-		|| rowsPerMeasure < rowsPerBeat
-		|| rowsPerMeasure > GetSoundFile().GetModSpecifications().patternRowsMax)
-	{
+	if(m_ModCommands.empty() || !IsValidRow(row))
 		return false;
+
+	for(const auto &m : GetRow(row))
+	{
+		if(m.command == CMD_PATTERNBREAK || m.command == CMD_POSITIONJUMP)
+			return true;
 	}
+	return false;
+}
+
+
+bool CPattern::SetSignature(const ROWINDEX rowsPerBeat, const ROWINDEX rowsPerMeasure) noexcept
+{
+	if(!IsValidSignature(rowsPerBeat, rowsPerMeasure))
+		return false;
 	m_RowsPerBeat = rowsPerBeat;
 	m_RowsPerMeasure = rowsPerMeasure;
 	return true;
 }
 
 
+bool CPattern::IsValidSignature(const ROWINDEX rowsPerBeat, const ROWINDEX rowsPerMeasure) noexcept
+{
+	return rowsPerBeat > 0 && rowsPerBeat <= MAX_ROWS_PER_BEAT
+	   && rowsPerBeat <= rowsPerMeasure && rowsPerMeasure <= MAX_ROWS_PER_MEASURE;
+}
+
+
 // Add or remove rows from the pattern.
 bool CPattern::Resize(const ROWINDEX newRowCount, bool enforceFormatLimits, bool resizeAtEnd)
 {
@@ -86,14 +100,14 @@ bool CPattern::Resize(const ROWINDEX newRowCount, bool enforceFormatLimits, bool
 		size_t count = ((newRowCount > m_Rows) ? (newRowCount - m_Rows) : (m_Rows - newRowCount)) * GetNumChannels();
 
 		if(newRowCount > m_Rows)
-			m_ModCommands.insert(resizeAtEnd ? m_ModCommands.end() : m_ModCommands.begin(), count, ModCommand::Empty());
+			m_ModCommands.insert(resizeAtEnd ? m_ModCommands.end() : m_ModCommands.begin(), count, ModCommand{});
 		else if(resizeAtEnd)
 			m_ModCommands.erase(m_ModCommands.end() - count, m_ModCommands.end());
 		else
 			m_ModCommands.erase(m_ModCommands.begin(), m_ModCommands.begin() + count);
-	} MPT_EXCEPTION_CATCH_OUT_OF_MEMORY(e)
+	} catch(mpt::out_of_memory e)
 	{
-		MPT_EXCEPTION_DELETE_OUT_OF_MEMORY(e);
+		mpt::delete_out_of_memory(e);
 		return false;
 	}
 
@@ -102,9 +116,9 @@ bool CPattern::Resize(const ROWINDEX newRowCount, bool enforceFormatLimits, bool
 }
 
 
-void CPattern::ClearCommands()
+void CPattern::ClearCommands() noexcept
 {
-	std::fill(m_ModCommands.begin(), m_ModCommands.end(), ModCommand::Empty());
+	std::fill(m_ModCommands.begin(), m_ModCommands.end(), ModCommand{});
 }
 
 
@@ -122,7 +136,7 @@ bool CPattern::AllocatePattern(ROWINDEX rows)
 	} else
 	{
 		// Do this in two steps in order to keep the old pattern data in case of OOM
-		decltype(m_ModCommands) newPattern(newSize, ModCommand::Empty());
+		decltype(m_ModCommands) newPattern(newSize, ModCommand{});
 		m_ModCommands = std::move(newPattern);
 	}
 	m_Rows = rows;
@@ -134,24 +148,36 @@ void CPattern::Deallocate()
 {
 	m_Rows = m_RowsPerBeat = m_RowsPerMeasure = 0;
 	m_ModCommands.clear();
+	m_tempoSwing.clear();
 	m_PatternName.clear();
 }
 
 
 CPattern& CPattern::operator= (const CPattern &pat)
 {
+	if(GetNumChannels() != pat.GetNumChannels())
+		return *this;
+
 	m_ModCommands = pat.m_ModCommands;
 	m_Rows = pat.m_Rows;
 	m_RowsPerBeat = pat.m_RowsPerBeat;
 	m_RowsPerMeasure = pat.m_RowsPerMeasure;
 	m_tempoSwing = pat.m_tempoSwing;
 	m_PatternName = pat.m_PatternName;
+
+	if(GetSoundFile().GetType() != pat.GetSoundFile().GetType())
+	{
+		for(ModCommand &m : m_ModCommands)
+		{
+			m.Convert(GetSoundFile().GetType(), pat.GetSoundFile().GetType(), GetSoundFile());
+		}
+	}
 	return *this;
 }
 
 
 
-bool CPattern::operator== (const CPattern &other) const
+bool CPattern::operator== (const CPattern &other) const noexcept
 {
 	return GetNumRows() == other.GetNumRows()
 		&& GetNumChannels() == other.GetNumChannels()
@@ -179,10 +205,10 @@ bool CPattern::Expand()
 	decltype(m_ModCommands) newPattern;
 	try
 	{
-		newPattern.assign(m_ModCommands.size() * 2, ModCommand::Empty());
-	} MPT_EXCEPTION_CATCH_OUT_OF_MEMORY(e)
+		newPattern.assign(m_ModCommands.size() * 2, ModCommand{});
+	} catch(mpt::out_of_memory e)
 	{
-		MPT_EXCEPTION_DELETE_OUT_OF_MEMORY(e);
+		mpt::delete_out_of_memory(e);
 		return false;
 	}
 
@@ -211,9 +237,9 @@ bool CPattern::Shrink()
 
 	for(ROWINDEX y = 0; y < m_Rows; y++)
 	{
-		const PatternRow srcRow = GetRow(y * 2);
-		const PatternRow nextSrcRow = GetRow(y * 2 + 1);
-		PatternRow destRow = GetRow(y);
+		const auto srcRow = GetRow(y * 2);
+		const auto nextSrcRow = GetRow(y * 2 + 1);
+		auto destRow = GetRow(y);
 
 		for(CHANNELINDEX x = 0; x < nChns; x++)
 		{
@@ -249,9 +275,9 @@ bool CPattern::Shrink()
 #endif // MODPLUG_TRACKER
 
 
-bool CPattern::SetName(const std::string &newName)
+bool CPattern::SetName(std::string newName)
 {
-	m_PatternName = newName;
+	m_PatternName = std::move(newName);
 	return true;
 }
 
@@ -262,7 +288,8 @@ bool CPattern::SetName(const char *newName, size_t maxChars)
 	{
 		return false;
 	}
-	m_PatternName = mpt::truncate(newName, maxChars);
+	const auto nameEnd = std::find(newName, newName + maxChars, '\0');
+	m_PatternName.assign(newName, nameEnd);
 	return true;
 }
 
@@ -355,9 +382,9 @@ bool CPattern::WriteEffect(EffectWriter &settings)
 					m->command = settings.m_command;
 
 					if(isS3M)
-						m->vol = (m->param + 1u) / 2u;
+						m->vol = static_cast<ModCommand::VOL>((m->param + 1u) / 2u);
 					else
-						m->vol = (m->param + 2u) / 4u;
+						m->vol = static_cast<ModCommand::VOL>((m->param + 2u) / 4u);
 
 					m->param = settings.m_param;
 					return true;
@@ -396,8 +423,8 @@ bool CPattern::WriteEffect(EffectWriter &settings)
 		} else
 		{
 			// Convert normal effect to volume effect
-			ModCommand::VOLCMD newVolCmd = VOLCMD_NONE;
-			ModCommand::VOL newVol = settings.m_param;
+			VolumeCommand newVolCmd = VOLCMD_NONE;
+			ModCommand::VOL newVol = 0;
 			if(settings.m_command == CMD_PANNING8 && isS3M)
 			{
 				// This needs some manual fixing.
@@ -405,21 +432,16 @@ bool CPattern::WriteEffect(EffectWriter &settings)
 				{
 					// Can't have surround in volume column, only normal panning
 					newVolCmd = VOLCMD_PANNING;
-					newVol /= 2u;
+					newVol = settings.m_param / 2u;
 				}
 			} else
 			{
-				newVolCmd = settings.m_command;
-				if(!ModCommand::ConvertVolEffect(newVolCmd, newVol, true))
-				{
-					// No Success :(
-					newVolCmd = VOLCMD_NONE;
-				}
+				std::tie(newVolCmd, newVol) = ModCommand::ConvertToVolCommand(settings.m_command, settings.m_param, true);
 			}
 
-			if(newVolCmd != CMD_NONE)
+			if(newVolCmd != VOLCMD_NONE)
 			{
-				settings.m_volcmd = static_cast<VolumeCommand>(newVolCmd);
+				settings.m_volcmd = newVolCmd;
 				settings.m_vol = newVol;
 				settings.m_retry = false;
 			}
@@ -496,17 +518,20 @@ void ReadModPattern(std::istream& iStrm, CPattern& pat, const size_t)
 {
 	srlztn::SsbRead ssb(iStrm);
 	ssb.BeginRead(FileIdPattern, Version::Current().GetRawVersion());
-	if ((ssb.GetStatus() & srlztn::SNT_FAILURE) != 0)
+	if(ssb.HasFailed())
+	{
 		return;
+	}
 	ssb.ReadItem(pat, "data", &ReadData);
 	// pattern time signature
-	uint32 nRPB = 0, nRPM = 0;
-	ssb.ReadItem<uint32>(nRPB, "RPB.");
-	ssb.ReadItem<uint32>(nRPM, "RPM.");
-	pat.SetSignature(nRPB, nRPM);
+	uint32 rpb = 0, rpm = 0;
+	ssb.ReadItem<uint32>(rpb, "RPB.");
+	ssb.ReadItem<uint32>(rpm, "RPM.");
+	pat.SetSignature(rpb, rpm);
 	TempoSwing swing;
 	ssb.ReadItem<TempoSwing>(swing, "SWNG", TempoSwing::Deserialize);
-	if(!swing.empty()) swing.resize(nRPB);
+	if(!swing.empty())
+		swing.resize(pat.GetRowsPerBeat());
 	pat.SetTempoSwing(swing);
 }
 
@@ -573,12 +598,12 @@ void WriteData(std::ostream& oStrm, const CPattern& pat)
 }
 
 
-#define READITEM(itembit,id)		\
+#define READITEM(itembit,id, type)	\
 if(diffmask & itembit)				\
 {									\
 	mpt::IO::ReadIntLE<uint8>(iStrm, temp);	\
 	if(ch < chns)					\
-		lastChnMC[ch].id = temp;	\
+		lastChnMC[ch].id = static_cast<type>(temp);	\
 }									\
 if(ch < chns)						\
 	m.id = lastChnMC[ch].id;
@@ -614,15 +639,15 @@ void ReadData(std::istream& iStrm, CPattern& pat, const size_t)
 			mpt::IO::ReadIntLE<uint8>(iStrm, diffmask);
 		uint8 temp = 0;
 
-		ModCommand dummy = ModCommand::Empty();
-		ModCommand& m = (ch < chns) ? *pat.GetpModCommand(row, ch) : dummy;
+		ModCommand dummy{};
+		ModCommand &m = (ch < chns) ? *pat.GetpModCommand(row, ch) : dummy;
 
-		READITEM(noteBit, note);
-		READITEM(instrBit, instr);
-		READITEM(volcmdBit, volcmd);
-		READITEM(volBit, vol);
-		READITEM(commandBit, command);
-		READITEM(effectParamBit, param);
+		READITEM(noteBit, note, ModCommand::NOTE);
+		READITEM(instrBit, instr, ModCommand::INSTR);
+		READITEM(volcmdBit, volcmd, ModCommand::VOLCMD);
+		READITEM(volBit, vol, ModCommand::VOL);
+		READITEM(commandBit, command, ModCommand::COMMAND);
+		READITEM(effectParamBit, param, ModCommand::PARAM);
 		if(diffmask & extraData)
 		{
 			//Ignore additional data.
@@ -630,6 +655,8 @@ void ReadData(std::istream& iStrm, CPattern& pat, const size_t)
 			mpt::IO::ReadIntLE<uint8>(iStrm, size);
 			iStrm.ignore(size);
 		}
+		if(!m.IsPcNote())
+			m.Clear();
 	}
 }
 
