@@ -27,10 +27,12 @@ public:
 	const CPattern& operator[](const int pat) const { return m_Patterns[pat]; }
 
 public:
-	CPatternContainer(CSoundFile& sndFile) : m_rSndFile(sndFile) { }
+	CPatternContainer(CSoundFile &sndFile) : m_rSndFile{sndFile} { }
+	CPatternContainer(const CPatternContainer &) = default;
+	CPatternContainer(CPatternContainer &&) noexcept = default;
+	CPatternContainer &operator=(const CPatternContainer &other);
+	CPatternContainer &operator=(CPatternContainer &&other) noexcept;
 
-	// Empty and initialize all patterns.
-	void ClearPatterns();
 	// Delete all patterns.
 	void DestroyPatterns();
 	
@@ -50,44 +52,45 @@ public:
 	//'invisible' - the pattern data is cleared but the actual pattern object won't get removed.
 	void Remove(const PATTERNINDEX index);
 
-	// Applies function object for modcommands in patterns in given range.
-	// Return: Copy of the function object.
+	// Applies function object for modcommands in all patterns.
 	template <class Func>
-	Func ForEachModCommand(PATTERNINDEX nStartPat, PATTERNINDEX nLastPat, Func func);
+	void ForEachModCommand(Func func);
 	template <class Func>
-	Func ForEachModCommand(Func func) { return ForEachModCommand(0, Size() - 1, func); }
+	void ForEachModCommand(Func func) const;
 
-	std::vector<CPattern>::iterator begin() { return m_Patterns.begin(); }
-	std::vector<CPattern>::const_iterator begin() const { return m_Patterns.begin(); }
-	std::vector<CPattern>::const_iterator cbegin() const { return m_Patterns.cbegin(); }
-	std::vector<CPattern>::iterator end() { return m_Patterns.end(); }
-	std::vector<CPattern>::const_iterator end() const { return m_Patterns.end(); }
-	std::vector<CPattern>::const_iterator cend() const { return m_Patterns.cend(); }
+	std::vector<CPattern>::iterator begin() noexcept { return m_Patterns.begin(); }
+	std::vector<CPattern>::const_iterator begin() const noexcept { return m_Patterns.begin(); }
+	std::vector<CPattern>::const_iterator cbegin() const noexcept { return m_Patterns.cbegin(); }
+	std::vector<CPattern>::iterator end() noexcept { return m_Patterns.end(); }
+	std::vector<CPattern>::const_iterator end() const noexcept { return m_Patterns.end(); }
+	std::vector<CPattern>::const_iterator cend() const noexcept { return m_Patterns.cend(); }
 
-	PATTERNINDEX Size() const { return static_cast<PATTERNINDEX>(m_Patterns.size()); }
+	PATTERNINDEX Size() const noexcept { return static_cast<PATTERNINDEX>(m_Patterns.size()); }
 
-	CSoundFile& GetSoundFile() { return m_rSndFile; }
-	const CSoundFile& GetSoundFile() const { return m_rSndFile; }
+	CSoundFile& GetSoundFile() noexcept { return m_rSndFile; }
+	const CSoundFile& GetSoundFile() const noexcept { return m_rSndFile; }
 
 	// Return true if pattern can be accessed with operator[](iPat), false otherwise.
-	bool IsValidIndex(const PATTERNINDEX iPat) const { return (iPat < Size()); }
+	bool IsValidIndex(const PATTERNINDEX iPat) const noexcept { return (iPat < Size()); }
 
 	// Return true if IsValidIndex() is true and the corresponding pattern has allocated modcommand array, false otherwise.
-	bool IsValidPat(const PATTERNINDEX iPat) const { return IsValidIndex(iPat) && m_Patterns[iPat].IsValid(); }
+	bool IsValidPat(const PATTERNINDEX iPat) const noexcept { return IsValidIndex(iPat) && m_Patterns[iPat].IsValid(); }
 
 	// Returns true if the pattern is empty, i.e. there are no notes/effects in this pattern
-	bool IsPatternEmpty(const PATTERNINDEX nPat) const;
+	bool IsPatternEmpty(const PATTERNINDEX nPat) const noexcept;
 	
 	void ResizeArray(const PATTERNINDEX newSize);
 
 	void OnModTypeChanged(const MODTYPE oldtype);
 
 	// Returns index of last valid pattern + 1, zero if no such pattern exists.
-	PATTERNINDEX GetNumPatterns() const;
+	PATTERNINDEX GetNumPatterns() const noexcept;
 
 	// Returns index of highest pattern with pattern named + 1.
-	PATTERNINDEX GetNumNamedPatterns() const;
+	PATTERNINDEX GetNumNamedPatterns() const noexcept;
 
+	// Number of patterns that can still be added, respecting the current format's limitations
+	PATTERNINDEX GetRemainingCapacity() const noexcept;
 
 private:
 	std::vector<CPattern> m_Patterns;
@@ -96,16 +99,20 @@ private:
 
 
 template <class Func>
-Func CPatternContainer::ForEachModCommand(PATTERNINDEX nStartPat, PATTERNINDEX nLastPat, Func func)
+void CPatternContainer::ForEachModCommand(Func func)
 {
-	if (nStartPat > nLastPat || nLastPat >= Size())
-		return func;
-	for (PATTERNINDEX nPat = nStartPat; nPat <= nLastPat; nPat++) if (m_Patterns[nPat].IsValid())
-		std::for_each(m_Patterns[nPat].begin(), m_Patterns[nPat].end(), func);
-	return func;
+	for(auto &pattern : m_Patterns)
+		std::for_each(pattern.begin(), pattern.end(), func);
 }
 
 
+template <class Func>
+void CPatternContainer::ForEachModCommand(Func func) const
+{
+	for(const auto &pattern : m_Patterns)
+		std::for_each(pattern.cbegin(), pattern.cend(), func);
+}
+
 const char FileIdPatterns[] = "mptPc";
 
 void ReadModPatterns(std::istream& iStrm, CPatternContainer& patc, const size_t nSize = 0);
