@@ -14,9 +14,11 @@
 #include "mpt/io/io_stdstream.hpp"
 #include "../common/serialization_utils.h"
 #include <algorithm>
-#include "../common/mptFileIO.h"
 #include "Loaders.h"
 #ifdef MODPLUG_TRACKER
+#include "../common/mptFileIO.h"
+#include "mpt/fs/fs.hpp"
+#include "mpt/io_file/outputfile.hpp"
 #include "../mptrack/TrackerSettings.h"
 #endif //MODPLUG_TRACKER
 
@@ -81,6 +83,26 @@ const CTuning* CTuningCollection::GetTuning(const mpt::ustring &name) const
 }
 
 
+const CTuning* CTuningCollection::FindIdenticalTuning(const CTuning &tuning) const
+{
+	auto result = std::find_if(m_Tunings.begin(), m_Tunings.end(), [&tuning](const std::unique_ptr<CTuning> &other)
+	{
+		return other && tuning == *other;
+	});
+	return (result != m_Tunings.end()) ? result->get() : nullptr;
+}
+
+
+CTuning* CTuningCollection::FindIdenticalTuning(const CTuning &tuning)
+{
+	auto result = std::find_if(m_Tunings.begin(), m_Tunings.end(), [&tuning](const std::unique_ptr<CTuning> &other)
+	{
+		return other && tuning == *other;
+	});
+	return (result != m_Tunings.end()) ? result->get() : nullptr;
+}
+
+
 Tuning::SerializationResult CTuningCollection::Serialize(std::ostream& oStrm, const mpt::ustring &name) const
 {
 	srlztn::SsbWrite ssb(oStrm);
@@ -95,7 +117,7 @@ Tuning::SerializationResult CTuningCollection::Serialize(std::ostream& oStrm, co
 		ssb.WriteItem(*m_Tunings[i], "2", &WriteTuning);
 	ssb.FinishWrite();
 		
-	if(ssb.GetStatus() & srlztn::SNT_FAILURE)
+	if(ssb.HasFailed())
 		return Tuning::SerializationResult::Failure;
 	else
 		return Tuning::SerializationResult::Success;
@@ -104,14 +126,14 @@ Tuning::SerializationResult CTuningCollection::Serialize(std::ostream& oStrm, co
 
 Tuning::SerializationResult CTuningCollection::Deserialize(std::istream &iStrm, mpt::ustring &name, mpt::Charset defaultCharset)
 {
-	std::istream::pos_type startpos = iStrm.tellg();
+	std::streamoff startpos = static_cast<std::streamoff>(iStrm.tellg());
 	
 	const Tuning::SerializationResult oldLoadingResult = DeserializeOLD(iStrm, name, defaultCharset);
 
 	if(oldLoadingResult == Tuning::SerializationResult::NoMagic)
 	{	// An old version was not recognised - trying new version.
 		iStrm.clear();
-		iStrm.seekg(startpos);
+		iStrm.seekg(startpos, std::ios::beg);
 		srlztn::SsbRead ssb(iStrm);
 		ssb.BeginRead("TC", 3); // version
 		int8 use_utf8 = 0;
@@ -123,15 +145,15 @@ Tuning::SerializationResult CTuningCollection::Deserialize(std::istream &iStrm,
 		for(srlztn::SsbRead::ReadIterator iter = iterBeg; iter != iterEnd; iter++)
 		{
 			uint16 dummyEditMask = 0xffff;
-			if (ssb.CompareId(iter, "0") == srlztn::SsbRead::IdMatch)
+			if(ssb.MatchesId(iter, "0"))
 				ssb.ReadIterItem(iter, name, [charset](std::istream &iStrm, mpt::ustring &ustr, const std::size_t dummy){ return ReadStr(iStrm, ustr, dummy, charset); });
-			else if (ssb.CompareId(iter, "1") == srlztn::SsbRead::IdMatch)
+			else if(ssb.MatchesId(iter, "1"))
 				ssb.ReadIterItem(iter, dummyEditMask);
-			else if (ssb.CompareId(iter, "2") == srlztn::SsbRead::IdMatch)
+			else if(ssb.MatchesId(iter, "2"))
 				ssb.ReadIterItem(iter, *this, [charset](std::istream &iStrm, CTuningCollection &Tc, const std::size_t dummy){ return ReadTuning(iStrm, Tc, dummy, charset); });
 		}
 
-		if(ssb.GetStatus() & srlztn::SNT_FAILURE)
+		if(ssb.HasFailed())
 			return Tuning::SerializationResult::Failure;
 		else
 			return Tuning::SerializationResult::Success;
@@ -281,7 +303,7 @@ CTuning* CTuningCollection::AddTuning(std::istream &inStrm, mpt::Charset default
 bool UnpackTuningCollection(const CTuningCollection &tc, const mpt::PathString &prefix)
 {
 	bool error = false;
-	auto numberFmt = mpt::FormatSpec().Dec().FillNul().Width(1 + static_cast<int>(std::log10(tc.GetNumTunings())));
+	auto numberFmt = mpt::format_simple_spec<mpt::ustring>().Dec().FillNul().Width(1 + static_cast<int>(std::log10(tc.GetNumTunings())));
 	for(std::size_t i = 0; i < tc.GetNumTunings(); ++i)
 	{
 		const CTuning & tuning = *(tc.GetTuning(i));
@@ -292,15 +314,14 @@ bool UnpackTuningCollection(const CTuningCollection &tc, const mpt::PathString &
 		{
 			tuningName = U_("untitled");
 		}
-		SanitizeFilename(tuningName);
-		fn += mpt::PathString::FromUnicode(MPT_UFORMAT("{} - {}")(mpt::ufmt::fmt(i + 1, numberFmt), tuningName));
+		fn += mpt::PathString::FromUnicode(MPT_UFORMAT("{} - {}")(mpt::ufmt::fmt(i + 1, numberFmt), tuningName)).AsSanitizedComponent();
 		fn += mpt::PathString::FromUTF8(CTuning::s_FileExtension);
-		if(fn.FileOrDirectoryExists())
+		if(mpt::native_fs{}.exists(fn))
 		{
 			error = true;
 		} else
 		{
-			mpt::SafeOutputFile sfout(fn, std::ios::binary, mpt::FlushModeFromBool(TrackerSettings::Instance().MiscFlushFileBuffersOnSave));
+			mpt::IO::SafeOutputFile sfout(fn, std::ios::binary, mpt::IO::FlushModeFromBool(TrackerSettings::Instance().MiscFlushFileBuffersOnSave));
 			if(tuning.Serialize(sfout) != Tuning::SerializationResult::Success)
 			{
 				error = true;
