@@ -17,6 +17,8 @@
 #include <ostream>
 #include <type_traits>
 
+#include <cstddef>
+
 
 
 namespace mpt {
@@ -28,15 +30,13 @@ namespace IO {
 
 
 
-//static_assert(sizeof(std::streamoff) == 8); // Assert 64bit file support.
-
 struct FileOperationsStdIos {
 
 private:
 	std::ios & f;
 
 public:
-	FileOperationsStdIos(std::ios & f_)
+	inline FileOperationsStdIos(std::ios & f_)
 		: f(f_) {
 		return;
 	}
@@ -54,7 +54,7 @@ private:
 	std::istream & f;
 
 public:
-	FileOperationsStdIstream(std::istream & f_)
+	inline FileOperationsStdIstream(std::istream & f_)
 		: FileOperationsStdIos(f_)
 		, f(f_) {
 		return;
@@ -64,30 +64,26 @@ public:
 	inline bool IsReadSeekable() {
 		f.clear();
 		std::streampos oldpos = f.tellg();
-		if (f.fail() || oldpos == std::streampos(-1))
-		{
+		if (f.fail() || oldpos == std::streampos(std::streamoff(-1))) {
 			f.clear();
 			return false;
 		}
 		f.seekg(0, std::ios::beg);
-		if (f.fail())
-		{
+		if (f.fail()) {
 			f.clear();
 			f.seekg(oldpos);
 			f.clear();
 			return false;
 		}
 		f.seekg(0, std::ios::end);
-		if (f.fail())
-		{
+		if (f.fail()) {
 			f.clear();
 			f.seekg(oldpos);
 			f.clear();
 			return false;
 		}
 		std::streampos length = f.tellg();
-		if (f.fail() || length == std::streampos(-1))
-		{
+		if (f.fail() || length == std::streampos(std::streamoff(-1))) {
 			f.clear();
 			f.seekg(oldpos);
 			f.clear();
@@ -99,11 +95,11 @@ public:
 	}
 
 	inline IO::Offset TellRead() {
-		return f.tellg();
+		return static_cast<std::streamoff>(f.tellg());
 	}
 
 	inline bool SeekBegin() {
-		f.seekg(0);
+		f.seekg(0, std::ios::beg);
 		return !f.fail();
 	}
 
@@ -113,8 +109,7 @@ public:
 	}
 
 	inline bool SeekAbsolute(IO::Offset pos) {
-		if (!mpt::in_range<std::streamoff>(pos))
-		{
+		if (!mpt::in_range<std::streamoff>(pos)) {
 			return false;
 		}
 		f.seekg(static_cast<std::streamoff>(pos), std::ios::beg);
@@ -122,8 +117,7 @@ public:
 	}
 
 	inline bool SeekRelative(IO::Offset off) {
-		if (!mpt::in_range<std::streamoff>(off))
-		{
+		if (!mpt::in_range<std::streamoff>(off)) {
 			return false;
 		}
 		f.seekg(static_cast<std::streamoff>(off), std::ios::cur);
@@ -131,8 +125,19 @@ public:
 	}
 
 	inline mpt::byte_span ReadRawImpl(mpt::byte_span data) {
-		f.read(mpt::byte_cast<char *>(data.data()), data.size());
-		return data.first(mpt::saturate_cast<std::size_t>(f.gcount()));
+		std::size_t bytesToRead = data.size();
+		std::size_t bytesRead = 0;
+		while (bytesToRead > 0) {
+			std::streamsize bytesChunkToRead = mpt::saturate_cast<std::streamsize>(bytesToRead);
+			f.read(mpt::byte_cast<char *>(data.data()) + bytesRead, bytesChunkToRead);
+			std::streamsize bytesChunkRead = f.gcount();
+			bytesRead += static_cast<std::size_t>(bytesChunkRead);
+			bytesToRead -= static_cast<std::size_t>(bytesChunkRead);
+			if (bytesChunkRead != bytesChunkToRead) {
+				break;
+			}
+		}
+		return data.first(bytesRead);
 	}
 
 	inline bool IsEof() {
@@ -147,7 +152,7 @@ private:
 	std::ostream & f;
 
 public:
-	FileOperationsStdOstream(std::ostream & f_)
+	inline FileOperationsStdOstream(std::ostream & f_)
 		: FileOperationsStdIos(f_)
 		, f(f_) {
 		return;
@@ -157,30 +162,26 @@ public:
 	inline bool IsWriteSeekable() {
 		f.clear();
 		std::streampos oldpos = f.tellp();
-		if (f.fail() || oldpos == std::streampos(-1))
-		{
+		if (f.fail() || oldpos == std::streampos(std::streamoff(-1))) {
 			f.clear();
 			return false;
 		}
 		f.seekp(0, std::ios::beg);
-		if (f.fail())
-		{
+		if (f.fail()) {
 			f.clear();
 			f.seekp(oldpos);
 			f.clear();
 			return false;
 		}
 		f.seekp(0, std::ios::end);
-		if (f.fail())
-		{
+		if (f.fail()) {
 			f.clear();
 			f.seekp(oldpos);
 			f.clear();
 			return false;
 		}
 		std::streampos length = f.tellp();
-		if (f.fail() || length == std::streampos(-1))
-		{
+		if (f.fail() || length == std::streampos(std::streamoff(-1))) {
 			f.clear();
 			f.seekp(oldpos);
 			f.clear();
@@ -192,11 +193,11 @@ public:
 	}
 
 	inline IO::Offset TellWrite() {
-		return f.tellp();
+		return static_cast<std::streamoff>(f.tellp());
 	}
 
 	inline bool SeekBegin() {
-		f.seekp(0);
+		f.seekp(0, std::ios::beg);
 		return !f.fail();
 	}
 
@@ -206,8 +207,7 @@ public:
 	}
 
 	inline bool SeekAbsolute(IO::Offset pos) {
-		if (!mpt::in_range<std::streamoff>(pos))
-		{
+		if (!mpt::in_range<std::streamoff>(pos)) {
 			return false;
 		}
 		f.seekp(static_cast<std::streamoff>(pos), std::ios::beg);
@@ -215,8 +215,7 @@ public:
 	}
 
 	inline bool SeekRelative(IO::Offset off) {
-		if (!mpt::in_range<std::streamoff>(off))
-		{
+		if (!mpt::in_range<std::streamoff>(off)) {
 			return false;
 		}
 		f.seekp(static_cast<std::streamoff>(off), std::ios::cur);
@@ -224,7 +223,17 @@ public:
 	}
 
 	inline bool WriteRawImpl(mpt::const_byte_span data) {
-		f.write(mpt::byte_cast<const char *>(data.data()), data.size());
+		std::size_t bytesToWrite = data.size();
+		std::size_t bytesWritten = 0;
+		while (bytesToWrite > 0) {
+			std::streamsize bytesChunk = mpt::saturate_cast<std::streamsize>(bytesToWrite);
+			f.write(mpt::byte_cast<const char *>(data.data()) + bytesWritten, bytesChunk);
+			if (f.fail()) {
+				break;
+			}
+			bytesWritten += static_cast<std::size_t>(bytesChunk);
+			bytesToWrite -= static_cast<std::size_t>(bytesChunk);
+		}
 		return !f.fail();
 	}
 
@@ -242,7 +251,7 @@ private:
 	std::iostream & f;
 
 public:
-	FileOperationsStdIOstream(std::iostream & f_)
+	inline FileOperationsStdIOstream(std::iostream & f_)
 		: FileOperationsStdIstream(f_)
 		, FileOperationsStdOstream(f_)
 		, f(f_) {
@@ -250,21 +259,23 @@ public:
 	}
 
 public:
+	// cppcheck-suppress duplInheritedMember
 	inline bool SeekBegin() {
 		FileOperationsStdIstream::SeekBegin();
 		FileOperationsStdOstream::SeekBegin();
 		return !f.fail();
 	}
 
+	// cppcheck-suppress duplInheritedMember
 	inline bool SeekEnd() {
 		FileOperationsStdIstream::SeekEnd();
 		FileOperationsStdOstream::SeekEnd();
 		return !f.fail();
 	}
 
+	// cppcheck-suppress duplInheritedMember
 	inline bool SeekAbsolute(IO::Offset pos) {
-		if (!mpt::in_range<std::streamoff>(pos))
-		{
+		if (!mpt::in_range<std::streamoff>(pos)) {
 			return false;
 		}
 		FileOperationsStdIstream::SeekAbsolute(pos);
@@ -272,9 +283,9 @@ public:
 		return !f.fail();
 	}
 
+	// cppcheck-suppress duplInheritedMember
 	inline bool SeekRelative(IO::Offset off) {
-		if (!mpt::in_range<std::streamoff>(off))
-		{
+		if (!mpt::in_range<std::streamoff>(off)) {
 			return false;
 		}
 		FileOperationsStdIstream::SeekRelative(off);
@@ -289,7 +300,7 @@ template <typename Tstream>
 struct FileOperations<Tstream, typename std::enable_if_t<std::is_base_of<std::iostream, Tstream>::value>>
 	: public FileOperationsStdIOstream {
 public:
-	FileOperations(Tstream & f)
+	inline FileOperations(Tstream & f)
 		: FileOperationsStdIOstream(f) {
 		return;
 	}
@@ -301,7 +312,7 @@ template <typename Tstream>
 struct FileOperations<Tstream, typename std::enable_if_t<std::is_base_of<std::istream, Tstream>::value && !std::is_base_of<std::iostream, Tstream>::value>>
 	: public FileOperationsStdIstream {
 public:
-	FileOperations(Tstream & f)
+	inline FileOperations(Tstream & f)
 		: FileOperationsStdIstream(f) {
 		return;
 	}
@@ -313,7 +324,7 @@ template <typename Tstream>
 struct FileOperations<Tstream, typename std::enable_if_t<std::is_base_of<std::ostream, Tstream>::value && !std::is_base_of<std::iostream, Tstream>::value>>
 	: public FileOperationsStdOstream {
 public:
-	FileOperations(Tstream & f)
+	inline FileOperations(Tstream & f)
 		: FileOperationsStdOstream(f) {
 		return;
 	}
