@@ -11,35 +11,55 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 
 #ifdef ENABLE_TESTS
 #ifdef MODPLUG_TRACKER
 
 
+#include "mpt/test/test.hpp"
+
+
 OPENMPT_NAMESPACE_BEGIN
 
-	
+
 namespace Test {
 
 
 #if MPT_COMPILER_MSVC
 // With MSVC, break directly using __debugbreak intrinsic instead of calling DebugBreak which breaks one stackframe deeper than we want
 #define MyDebugBreak() __debugbreak()
+#elif MPT_COMPILER_CLANG
+#define MyDebugBreak() __builtin_debugtrap()
+#elif MPT_COMPILER_GCC && (MPT_ARCH_X86 || MPT_ARCH_AMD64)
+#define MyDebugBreak() __asm__ __volatile__("int 3");
 #else
 #define MyDebugBreak() DebugBreak()
 #endif
 
 
+class mpt_test_reporter
+	: public mpt::test::silent_reporter
+{
+public:
+	mpt_test_reporter() = default;
+	~mpt_test_reporter() override = default;
+public:
+	inline void immediate_breakpoint() override {
+		MyDebugBreak();
+	}
+};
+
+
 // Verify that given parameters are 'equal'. Break directly into the debugger if not.
-// The exact meaning of equality is based on operator!= .
+// The exact meaning of equality is based on operator== of the compared types.
 #define VERIFY_EQUAL(x,y)	\
-	MPT_DO { \
-		if((x) != (y)) { \
+	do { \
+		if(!((x) == (y))) { \
 			MyDebugBreak(); \
 		} \
-	} MPT_WHILE_0 \
+	} while(0) \
 /**/
 
 // Like VERIFY_EQUAL, only differs for libopenmpt
@@ -49,20 +69,20 @@ namespace Test {
 #define VERIFY_EQUAL_QUIET_NONCONT VERIFY_EQUAL
 
 #define VERIFY_EQUAL_EPS(x,y,eps)	\
-	MPT_DO { \
-		if(mpt::abs((x) - (y)) > (eps)) { \
+	do { \
+		if(std::abs((x) - (y)) > (eps)) { \
 			MyDebugBreak(); \
 		} \
-	} MPT_WHILE_0 \
+	} while(0) \
 /**/
 
 
 #define DO_TEST(func) \
-	MPT_DO { \
+	do { \
 		if(IsDebuggerPresent()) { \
 			func(); \
 		} \
-	} MPT_WHILE_0 \
+	} while(0) \
 /**/
 
 
